"""
Created on Apr 18, 2017

@author: Christopher Bruns
"""

import math

import numpy


def pack(matrix, do_transpose=False):
    if do_transpose:
        return numpy.ascontiguousarray(matrix.T)
    else:
        return numpy.ascontiguousarray(matrix)


def frustum(left, right, bottom, top, z_near, z_far):
    a = (right + left) / (right - left)
    b = (top + bottom) / (top - bottom)
    c = -(z_far + z_near) / (z_far - z_near)
    d = -(2.0 * z_far * z_near) / (z_far - z_near)
    return numpy.matrix([
            [2.0 * z_near / (right - left), 0.0, a, 0.0],
            [0.0, 2.0 * z_near / (top - bottom), b, 0.0],
            [0.0, 0.0, c, d],
            [0.0, 0.0, -1.0, 0.0]], dtype=numpy.float32).T


def identity():
    return numpy.matrix([
            [1, 0, 0, 0],
            [0, 1, 0, 0],
            [0, 0, 1, 0],
            [0, 0, 0, 1]], dtype=numpy.float32)


def perspective(fov_y, aspect, z_near, z_far):
    f_h = math.tan(fov_y / 2.0 / 180.0 * math.pi) * z_near
    f_w = f_h * aspect
    return frustum(-f_w, f_w, -f_h, f_h, z_near, z_far)


def rotate_x(angle):
    s = math.sin(float(angle))
    c = math.cos(float(angle))
    return numpy.matrix([
            [1, 0, 0, 0],
            [0, c, -s, 0],
            [0, s, c, 0],
            [0, 0, 0, 1]], dtype=numpy.float32).T


def rotate_y(angle):
    s = math.sin(float(angle))
    c = math.cos(float(angle))
    return numpy.matrix([
            [c, 0, s, 0],
            [0, 1, 0, 0],
            [-s, 0, c, 0],
            [0, 0, 0, 1]], dtype=numpy.float32).T


def rotate_z(angle):
    s = math.sin(float(angle))
    c = math.cos(float(angle))
    return numpy.matrix([
            [c, -s, 0, 0],
            [s, c, 0, 0],
            [0, 0, 1, 0],
            [0, 0, 0, 1]], dtype=numpy.float32).T


def scale(sx, sy=None, sz=None):
    if sy is None:
        sy = sx
    if sz is None:
        sz = sx
    return numpy.matrix([
                         (sx, 0, 0, 0),
                         (0, sy, 0, 0),
                         (0, 0, sz, 0),
                         (0, 0, 0, 1)], dtype=numpy.float32)


def translate(xyz):
    x, y, z = xyz
    array = [
            [1, 0, 0, x],
            [0, 1, 0, y],
            [0, 0, 1, z],
            [0, 0, 0, 1]]
    mat = numpy.matrix(array, dtype=numpy.float32)
    return mat.T





#!/bin/env python

# file tracked_devices_actor.py

import time
from ctypes import cast, c_float, c_void_p, sizeof

import numpy
from OpenGL import GL
from OpenGL.GL.shaders import compileShader, compileProgram
from OpenGL.arrays import vbo
from OpenGL.GL.EXT.texture_filter_anisotropic import GL_TEXTURE_MAX_ANISOTROPY_EXT, GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT

import openvr
from openvr.gl_renderer import matrixForOpenVrMatrix
from openvr.glframework import shader_string

"""
Tracked item (controllers, lighthouses, etc) actor for "hello world" openvr apps
"""


class TrackedDeviceMesh(object):
    def __init__(self, model_name):
        """This constructor must only be called with a live OpenGL context"""
        self.model_name = model_name
        self.model_is_loaded = False
        self.texture_is_loaded = False
        self.vao = None
        self.vbo = None
        self._try_load_model()
        self.vertexPositions = None

    def _try_load_model(self):
        try:
            model = openvr.VRRenderModels().loadRenderModel_Async(self.model_name)
        except openvr.error_code.RenderModelError_Loading:
            return
        vertices0 = list()
        indices0 = list()
        if model is not None:
            for v in range(model.unVertexCount):
                vd = model.rVertexData[v]
                vertices0.append(float(vd.vPosition.v[0]))  # position X
                vertices0.append(float(vd.vPosition.v[1]))  # position Y
                vertices0.append(float(vd.vPosition.v[2]))  # position Z
                vertices0.append(float(vd.vNormal.v[0]))  # normal X
                vertices0.append(float(vd.vNormal.v[1]))  # normal Y
                vertices0.append(float(vd.vNormal.v[2]))  # normal Z
                vertices0.append(float(vd.rfTextureCoord[0]))  # texture coordinate U
                vertices0.append(float(vd.rfTextureCoord[1]))  # texture coordinate V
            for i in range(model.unTriangleCount * 3):
                index = model.rIndexData[i]
                indices0.append(int(index))
        vertices0 = numpy.array(vertices0, dtype=numpy.float32)
        indices0 = numpy.array(indices0, dtype=numpy.uint32)
        #
        self.vertexPositions = vbo.VBO(vertices0)
        self.indexPositions = vbo.VBO(indices0, target=GL.GL_ELEMENT_ARRAY_BUFFER)
        # http://stackoverflow.com/questions/14365484/how-to-draw-with-vertex-array-objects-and-gldrawelements-in-pyopengl
        self.vao = GL.glGenVertexArrays(1)
        GL.glBindVertexArray(self.vao)
        self.vertexPositions.bind()
        self.indexPositions.bind()
        # Vertices
        GL.glEnableVertexAttribArray(0)
        fsize = sizeof(c_float)
        GL.glVertexAttribPointer(0, 3, GL.GL_FLOAT, False, 8 * fsize, cast(0 * fsize, c_void_p))
        # Normals
        GL.glEnableVertexAttribArray(1)
        GL.glVertexAttribPointer(1, 3, GL.GL_FLOAT, False, 8 * fsize, cast(3 * fsize, c_void_p))
        # Texture coordinates    
        GL.glEnableVertexAttribArray(2)
        GL.glVertexAttribPointer(2, 2, GL.GL_FLOAT, False, 8 * fsize, cast(6 * fsize, c_void_p))
        GL.glBindVertexArray(0)
        self.model = model
        self.model_is_loaded = True
        self._try_load_texture()

    def _try_load_texture(self):
        # Surface texture
        try:
            texture_map = openvr.VRRenderModels().loadTexture_Async(self.model.diffuseTextureId)
        except openvr.error_code.RenderModelError_Loading:
            return
        self.texture_map = texture_map
        self.diffuse_texture = GL.glGenTextures(1)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.diffuse_texture)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGBA, self.texture_map.unWidth, self.texture_map.unHeight,
                        0, GL.GL_RGBA,
                        GL.GL_UNSIGNED_BYTE, self.texture_map.rubTextureMapData)
        GL.glGenerateMipmap(GL.GL_TEXTURE_2D)
        GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP_TO_EDGE)
        GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP_TO_EDGE)
        GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR)
        GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR_MIPMAP_LINEAR)
        fLargest = GL.glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT)
        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, fLargest)
        GL.glBindTexture(GL.GL_TEXTURE_2D, 0)
        self.texture_is_loaded = True

    def display_gl(self, modelview, projection, pose):
        if not self.model_is_loaded:
            self._try_load_model()
            return
        if not self.texture_is_loaded:
            self._try_load_texture()
            return
        controller_X_room = pose.mDeviceToAbsoluteTracking
        controller_X_room = matrixForOpenVrMatrix(controller_X_room)
        modelview0 = controller_X_room * modelview
        # Repack before use, just in case
        modelview0 = numpy.asarray(numpy.matrix(modelview0, dtype=numpy.float32))
        GL.glUniformMatrix4fv(4, 1, False, modelview0)
        normal_matrix = numpy.asarray(controller_X_room)
        GL.glUniformMatrix4fv(8, 1, False, normal_matrix)
        GL.glActiveTexture(GL.GL_TEXTURE0)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.diffuse_texture)
        GL.glBindVertexArray(self.vao)
        GL.glDrawElements(GL.GL_TRIANGLES, len(self.indexPositions), GL.GL_UNSIGNED_INT, None)
        GL.glBindVertexArray(0)

    def dispose_gl(self):
        if self.vao is not None:
            GL.glDeleteVertexArrays(1, (self.vao,))
            self.vao = None
        self.vbo = None
        if self.vertexPositions is not None:
            self.vertexPositions.delete()
            self.vertexPositions = None
            self.indexPositions.delete()
            self.indexPositions = None


class TrackedDevicesActor(object):
    """
    Draws Vive controllers and lighthouses.
    """

    def __init__(self, pose_array):
        self.shader = 0
        self.poses = pose_array
        self.meshes = dict()
        self.show_controllers_only = True

    def _check_devices(self):
        "Enumerate OpenVR tracked devices and check whether any need to be initialized"
        for i in range(1, len(self.poses)):
            pose = self.poses[i]
            if not pose.bDeviceIsConnected:
                continue
            if not pose.bPoseIsValid:
                continue
            if self.show_controllers_only:
                device_class = openvr.VRSystem().getTrackedDeviceClass(i)
                if not device_class == openvr.TrackedDeviceClass_Controller:
                    continue
            model_name = openvr.VRSystem().getStringTrackedDeviceProperty(i, openvr.Prop_RenderModelName_String)
            # Create a new mesh object, if necessary
            if model_name not in self.meshes:
                self.meshes[model_name] = TrackedDeviceMesh(model_name)

    def init_gl(self):
        vertex_shader = compileShader(
            shader_string("""
            layout(location = 0) in vec3 in_Position;
            layout(location = 1) in vec3 in_Normal;
            layout(location = 2) in vec2 in_TexCoord;
            
            layout(location = 0) uniform mat4 projection = mat4(1);
            layout(location = 4) uniform mat4 model_view = mat4(1);
            layout(location = 8) uniform mat4 normal_matrix = mat4(1);
            
            out vec3 color;
            out vec2 fragTexCoord;
            
            void main() {
              gl_Position = projection * model_view * vec4(in_Position, 1.0);
              vec3 normal = normalize((normal_matrix * vec4(in_Normal, 0)).xyz);
              color = (normal + vec3(1,1,1)) * 0.5; // color by normal
              fragTexCoord = in_TexCoord;
              // color = vec3(in_TexCoord, 0.5); // color by texture coordinate
            }
            """),
            GL.GL_VERTEX_SHADER)
        fragment_shader = compileShader(
            shader_string("""
            uniform sampler2D diffuse;
            in vec3 color;
            in vec2 fragTexCoord;
            out vec4 fragColor;
            
            void main() {
              // fragColor = vec4(color, 1.0);
              fragColor = texture(diffuse, fragTexCoord);
            }
            """),
            GL.GL_FRAGMENT_SHADER)
        self.shader = compileProgram(vertex_shader, fragment_shader)
        self._check_devices()
        GL.glEnable(GL.GL_DEPTH_TEST)

    def display_gl(self, modelview, projection):
        self._check_devices()
        GL.glEnable(GL.GL_DEPTH_TEST)
        GL.glUseProgram(self.shader)
        GL.glUniformMatrix4fv(0, 1, False, projection)
        for i in range(1, len(self.poses)):
            pose = self.poses[i]
            if not pose.bPoseIsValid:
                continue
            model_name = openvr.VRSystem().getStringTrackedDeviceProperty(i, openvr.Prop_RenderModelName_String)
            if model_name not in self.meshes:
                continue  # Come on, we already tried to load it a moment ago. Maybe next time.
            mesh = self.meshes[model_name]
            mesh.display_gl(modelview, projection, pose)

    def dispose_gl(self):
        GL.glDeleteProgram(self.shader)
        self.shader = 0
        for key in list(self.meshes):
            mesh = self.meshes[key]
            mesh.dispose_gl()
            del self.meshes[key]



#!/bin/env python

# file glfwapp.py

import glfw
from OpenGL.GL import glFlush


"""
Toy glfw application for use with "hello world" examples demonstrating pyopenvr
"""


class GlfwBaseApp(object):
    "Base version renders to desktop"

    def __init__(self, renderer, title="GlfwBaseApp"):
        "Creates an OpenGL context and a window, and acquires OpenGL resources"
        self.renderer = renderer
        self.title = title
        self._is_initialized = False # keep track of whether self.init_gl() has been called
        
        if not glfw.init():
            raise Exception("GLFW Initialization error")
        # Use modern OpenGL version 4.5 core
        glfw.window_hint(glfw.CONTEXT_VERSION_MAJOR, 4)
        glfw.window_hint(glfw.CONTEXT_VERSION_MINOR, 5)
        glfw.window_hint(glfw.OPENGL_PROFILE, glfw.OPENGL_CORE_PROFILE)
        self._configure_context()
        self.window = glfw.create_window(self.renderer.window_size[0], self.renderer.window_size[1], self.title, None, None)
        if self.window is None:
            glfw.terminate()
            raise Exception("GLFW window creation error")
        glfw.set_key_callback(self.window, self.key_callback)
        glfw.make_context_current(self.window)

    def _configure_context(self):
        pass

    def __enter__(self):
        "setup for RAII using 'with' keyword"
        return self

    def __exit__(self, type_arg, value, traceback):
        "cleanup for RAII using 'with' keyword"
        self.dispose_gl()

    def init_gl(self):
        if self._is_initialized:
            return # only initialize once
        glfw.make_context_current(self.window)
        if self.renderer:
            self.renderer.init_gl()
        self._is_initialized = True

    def render_scene(self):
        "render scene one time"
        self.init_gl() # should be a no-op after the first frame is rendered
        glfw.make_context_current(self.window)
        self.renderer.render_scene()
        # Done rendering
        # glfw.swap_buffers(self.window) # avoid double buffering to avoid stalling
        glFlush() # single buffering
        glfw.poll_events()

    def dispose_gl(self):
        if self.window:
            glfw.make_context_current(self.window)
            if self.renderer:
                self.renderer.dispose_gl()
        glfw.destroy_window(self.window)
        glfw.terminate()
        self._is_initialized = False

    def key_callback(self, window, key, scancode, action, mods):
        "press ESCAPE to quit the application"
        if key == glfw.KEY_ESCAPE and action == glfw.PRESS:
            glfw.set_window_should_close(self.window, True)

    def run_loop(self):
        "keep rendering until the user says quit"
        while not glfw.window_should_close(self.window):
            self.render_scene()


class GlfwApp(GlfwBaseApp):
    "GlfwApp uses glfw library to create an opengl context, listen to keyboard events, and clean up"

    def __init__(self, renderer, title="GlfwApp"):
        super(GlfwApp, self).__init__(renderer=renderer, title=title)
        glfw.swap_interval(0)

    def _configure_context(self):
        # Double buffered screen mirror stalls VR headset rendering,
        # So use single-buffering
        glfw.window_hint(glfw.DOUBLEBUFFER, False)


class GlfwVrApp(GlfwApp):
    def __init__(self, actors=[], title="GlfwVrApp"):
        import openvr.gl_renderer
        renderer = openvr.gl_renderer.OpenVrGlRenderer(actor=actors, multisample=2)
        super(GlfwVrApp, self).__init__(renderer=renderer)
        glfw.swap_interval(0)



############################################
# Python exceptions for openvr error codes #
############################################


class OpenVRError(Exception):
    """
    OpenVRError is a custom exception type for when OpenVR functions return a failure code.
    Such a specific exception type allows more precise exception handling that does just raising Exception().
    """
    pass


class ErrorCode(OpenVRError):
    error_index = dict()
    is_error = True  # FooError_None classes should override this

    @classmethod
    def check_error_value(cls, error_value, message=''):
        error_class = cls.error_index[int(error_value)]
        if not error_class.is_error:
            return
        raise error_class(error_value, message)
        
    def __init__(this, error_value, message=''):
        super().__init__(message)
        this.error_value = error_value


class BufferTooSmallError(ErrorCode):
    pass


class TrackedPropertyError(ErrorCode):
    error_index = dict()


class TrackedProp_Success(TrackedPropertyError):
    is_error = False


class TrackedProp_WrongDataType(TrackedPropertyError):
    pass


class TrackedProp_WrongDeviceClass(TrackedPropertyError):
    pass


class TrackedProp_BufferTooSmall(TrackedPropertyError, BufferTooSmallError):
    pass


class TrackedProp_UnknownProperty(TrackedPropertyError):
    pass


class TrackedProp_InvalidDevice(TrackedPropertyError):
    pass


class TrackedProp_CouldNotContactServer(TrackedPropertyError):
    pass


class TrackedProp_ValueNotProvidedByDevice(TrackedPropertyError):
    pass


class TrackedProp_StringExceedsMaximumLength(TrackedPropertyError):
    pass


class TrackedProp_NotYetAvailable(TrackedPropertyError):
    pass


class TrackedProp_PermissionDenied(TrackedPropertyError):
    pass


class TrackedProp_InvalidOperation(TrackedPropertyError):
    pass


class TrackedProp_CannotWriteToWildcards(TrackedPropertyError):
    pass


class TrackedProp_IPCReadFailure(TrackedPropertyError):
    pass


class TrackedProp_OutOfMemory(TrackedPropertyError):
    pass


class TrackedProp_InvalidContainer(TrackedPropertyError):
    pass


TrackedPropertyError.error_index[0] = TrackedProp_Success
TrackedPropertyError.error_index[1] = TrackedProp_WrongDataType
TrackedPropertyError.error_index[2] = TrackedProp_WrongDeviceClass
TrackedPropertyError.error_index[3] = TrackedProp_BufferTooSmall
TrackedPropertyError.error_index[4] = TrackedProp_UnknownProperty
TrackedPropertyError.error_index[5] = TrackedProp_InvalidDevice
TrackedPropertyError.error_index[6] = TrackedProp_CouldNotContactServer
TrackedPropertyError.error_index[7] = TrackedProp_ValueNotProvidedByDevice
TrackedPropertyError.error_index[8] = TrackedProp_StringExceedsMaximumLength
TrackedPropertyError.error_index[9] = TrackedProp_NotYetAvailable
TrackedPropertyError.error_index[10] = TrackedProp_PermissionDenied
TrackedPropertyError.error_index[11] = TrackedProp_InvalidOperation
TrackedPropertyError.error_index[12] = TrackedProp_CannotWriteToWildcards
TrackedPropertyError.error_index[13] = TrackedProp_IPCReadFailure
TrackedPropertyError.error_index[14] = TrackedProp_OutOfMemory
TrackedPropertyError.error_index[15] = TrackedProp_InvalidContainer


class HDCPError(ErrorCode):
    error_index = dict()


class HDCPError_None(HDCPError):
    is_error = False


class HDCPError_LinkLost(HDCPError):
    pass


class HDCPError_Tampered(HDCPError):
    pass


class HDCPError_DeviceRevoked(HDCPError):
    pass


class HDCPError_Unknown(HDCPError):
    pass


HDCPError.error_index[0] = HDCPError_None
HDCPError.error_index[1] = HDCPError_LinkLost
HDCPError.error_index[2] = HDCPError_Tampered
HDCPError.error_index[3] = HDCPError_DeviceRevoked
HDCPError.error_index[4] = HDCPError_Unknown


class InputError(ErrorCode):
    error_index = dict()


class InputError_None(InputError):
    is_error = False


class InputError_NameNotFound(InputError):
    pass


class InputError_WrongType(InputError):
    pass


class InputError_InvalidHandle(InputError):
    pass


class InputError_InvalidParam(InputError):
    pass


class InputError_NoSteam(InputError):
    pass


class InputError_MaxCapacityReached(InputError):
    pass


class InputError_IPCError(InputError):
    pass


class InputError_NoActiveActionSet(InputError):
    pass


class InputError_InvalidDevice(InputError):
    pass


class InputError_InvalidSkeleton(InputError):
    pass


class InputError_InvalidBoneCount(InputError):
    pass


class InputError_InvalidCompressedData(InputError):
    pass


class InputError_NoData(InputError):
    pass


class InputError_BufferTooSmall(InputError, BufferTooSmallError):
    pass


class InputError_MismatchedActionManifest(InputError):
    pass


class InputError_MissingSkeletonData(InputError):
    pass


class InputError_InvalidBoneIndex(InputError):
    pass


class InputError_InvalidPriority(InputError):
    pass


class InputError_PermissionDenied(InputError):
    pass


class InputError_InvalidRenderModel(InputError):
    pass


InputError.error_index[0] = InputError_None
InputError.error_index[1] = InputError_NameNotFound
InputError.error_index[2] = InputError_WrongType
InputError.error_index[3] = InputError_InvalidHandle
InputError.error_index[4] = InputError_InvalidParam
InputError.error_index[5] = InputError_NoSteam
InputError.error_index[6] = InputError_MaxCapacityReached
InputError.error_index[7] = InputError_IPCError
InputError.error_index[8] = InputError_NoActiveActionSet
InputError.error_index[9] = InputError_InvalidDevice
InputError.error_index[10] = InputError_InvalidSkeleton
InputError.error_index[11] = InputError_InvalidBoneCount
InputError.error_index[12] = InputError_InvalidCompressedData
InputError.error_index[13] = InputError_NoData
InputError.error_index[14] = InputError_BufferTooSmall
InputError.error_index[15] = InputError_MismatchedActionManifest
InputError.error_index[16] = InputError_MissingSkeletonData
InputError.error_index[17] = InputError_InvalidBoneIndex
InputError.error_index[18] = InputError_InvalidPriority
InputError.error_index[19] = InputError_PermissionDenied
InputError.error_index[20] = InputError_InvalidRenderModel


class SpatialAnchorError(ErrorCode):
    error_index = dict()


class SpatialAnchorError_Success(SpatialAnchorError):
    is_error = False


class SpatialAnchorError_Internal(SpatialAnchorError):
    pass


class SpatialAnchorError_UnknownHandle(SpatialAnchorError):
    pass


class SpatialAnchorError_ArrayTooSmall(SpatialAnchorError):
    pass


class SpatialAnchorError_InvalidDescriptorChar(SpatialAnchorError):
    pass


class SpatialAnchorError_NotYetAvailable(SpatialAnchorError):
    pass


class SpatialAnchorError_NotAvailableInThisUniverse(SpatialAnchorError):
    pass


class SpatialAnchorError_PermanentlyUnavailable(SpatialAnchorError):
    pass


class SpatialAnchorError_WrongDriver(SpatialAnchorError):
    pass


class SpatialAnchorError_DescriptorTooLong(SpatialAnchorError):
    pass


class SpatialAnchorError_Unknown(SpatialAnchorError):
    pass


class SpatialAnchorError_NoRoomCalibration(SpatialAnchorError):
    pass


class SpatialAnchorError_InvalidArgument(SpatialAnchorError):
    pass


class SpatialAnchorError_UnknownDriver(SpatialAnchorError):
    pass


SpatialAnchorError.error_index[0] = SpatialAnchorError_Success
SpatialAnchorError.error_index[1] = SpatialAnchorError_Internal
SpatialAnchorError.error_index[2] = SpatialAnchorError_UnknownHandle
SpatialAnchorError.error_index[3] = SpatialAnchorError_ArrayTooSmall
SpatialAnchorError.error_index[4] = SpatialAnchorError_InvalidDescriptorChar
SpatialAnchorError.error_index[5] = SpatialAnchorError_NotYetAvailable
SpatialAnchorError.error_index[6] = SpatialAnchorError_NotAvailableInThisUniverse
SpatialAnchorError.error_index[7] = SpatialAnchorError_PermanentlyUnavailable
SpatialAnchorError.error_index[8] = SpatialAnchorError_WrongDriver
SpatialAnchorError.error_index[9] = SpatialAnchorError_DescriptorTooLong
SpatialAnchorError.error_index[10] = SpatialAnchorError_Unknown
SpatialAnchorError.error_index[11] = SpatialAnchorError_NoRoomCalibration
SpatialAnchorError.error_index[12] = SpatialAnchorError_InvalidArgument
SpatialAnchorError.error_index[13] = SpatialAnchorError_UnknownDriver


class OverlayError(ErrorCode):
    error_index = dict()


class OverlayError_None(OverlayError):
    is_error = False


class OverlayError_UnknownOverlay(OverlayError):
    pass


class OverlayError_InvalidHandle(OverlayError):
    pass


class OverlayError_PermissionDenied(OverlayError):
    pass


class OverlayError_OverlayLimitExceeded(OverlayError):
    pass


class OverlayError_WrongVisibilityType(OverlayError):
    pass


class OverlayError_KeyTooLong(OverlayError):
    pass


class OverlayError_NameTooLong(OverlayError):
    pass


class OverlayError_KeyInUse(OverlayError):
    pass


class OverlayError_WrongTransformType(OverlayError):
    pass


class OverlayError_InvalidTrackedDevice(OverlayError):
    pass


class OverlayError_InvalidParameter(OverlayError):
    pass


class OverlayError_ThumbnailCantBeDestroyed(OverlayError):
    pass


class OverlayError_ArrayTooSmall(OverlayError):
    pass


class OverlayError_RequestFailed(OverlayError):
    pass


class OverlayError_InvalidTexture(OverlayError):
    pass


class OverlayError_UnableToLoadFile(OverlayError):
    pass


class OverlayError_KeyboardAlreadyInUse(OverlayError):
    pass


class OverlayError_NoNeighbor(OverlayError):
    pass


class OverlayError_TooManyMaskPrimitives(OverlayError):
    pass


class OverlayError_BadMaskPrimitive(OverlayError):
    pass


class OverlayError_TextureAlreadyLocked(OverlayError):
    pass


class OverlayError_TextureLockCapacityReached(OverlayError):
    pass


class OverlayError_TextureNotLocked(OverlayError):
    pass


OverlayError.error_index[0] = OverlayError_None
OverlayError.error_index[10] = OverlayError_UnknownOverlay
OverlayError.error_index[11] = OverlayError_InvalidHandle
OverlayError.error_index[12] = OverlayError_PermissionDenied
OverlayError.error_index[13] = OverlayError_OverlayLimitExceeded
OverlayError.error_index[14] = OverlayError_WrongVisibilityType
OverlayError.error_index[15] = OverlayError_KeyTooLong
OverlayError.error_index[16] = OverlayError_NameTooLong
OverlayError.error_index[17] = OverlayError_KeyInUse
OverlayError.error_index[18] = OverlayError_WrongTransformType
OverlayError.error_index[19] = OverlayError_InvalidTrackedDevice
OverlayError.error_index[20] = OverlayError_InvalidParameter
OverlayError.error_index[21] = OverlayError_ThumbnailCantBeDestroyed
OverlayError.error_index[22] = OverlayError_ArrayTooSmall
OverlayError.error_index[23] = OverlayError_RequestFailed
OverlayError.error_index[24] = OverlayError_InvalidTexture
OverlayError.error_index[25] = OverlayError_UnableToLoadFile
OverlayError.error_index[26] = OverlayError_KeyboardAlreadyInUse
OverlayError.error_index[27] = OverlayError_NoNeighbor
OverlayError.error_index[29] = OverlayError_TooManyMaskPrimitives
OverlayError.error_index[30] = OverlayError_BadMaskPrimitive
OverlayError.error_index[31] = OverlayError_TextureAlreadyLocked
OverlayError.error_index[32] = OverlayError_TextureLockCapacityReached
OverlayError.error_index[33] = OverlayError_TextureNotLocked


class FirmwareError(ErrorCode):
    error_index = dict()


class FirmwareError_None(FirmwareError):
    is_error = False


class FirmwareError_Success(FirmwareError):
    is_error = False


class FirmwareError_Fail(FirmwareError):
    pass


FirmwareError.error_index[0] = FirmwareError_None
FirmwareError.error_index[1] = FirmwareError_Success
FirmwareError.error_index[2] = FirmwareError_Fail


class NotificationError(ErrorCode):
    error_index = dict()


class NotificationError_OK(NotificationError):
    pass


class NotificationError_InvalidNotificationId(NotificationError):
    pass


class NotificationError_NotificationQueueFull(NotificationError):
    pass


class NotificationError_InvalidOverlayHandle(NotificationError):
    pass


class NotificationError_SystemWithUserValueAlreadyExists(NotificationError):
    pass


NotificationError.error_index[0] = NotificationError_OK
NotificationError.error_index[100] = NotificationError_InvalidNotificationId
NotificationError.error_index[101] = NotificationError_NotificationQueueFull
NotificationError.error_index[102] = NotificationError_InvalidOverlayHandle
NotificationError.error_index[103] = NotificationError_SystemWithUserValueAlreadyExists


class InitError(ErrorCode):
    error_index = dict()


class InitError_None(InitError):
    is_error = False


class InitError_Unknown(InitError):
    pass


class InitError_Init_InstallationNotFound(InitError):
    pass


class InitError_Init_InstallationCorrupt(InitError):
    pass


class InitError_Init_VRClientDLLNotFound(InitError):
    pass


class InitError_Init_FileNotFound(InitError):
    pass


class InitError_Init_FactoryNotFound(InitError):
    pass


class InitError_Init_InterfaceNotFound(InitError):
    pass


class InitError_Init_InvalidInterface(InitError):
    pass


class InitError_Init_UserConfigDirectoryInvalid(InitError):
    pass


class InitError_Init_HmdNotFound(InitError):
    pass


class InitError_Init_NotInitialized(InitError):
    pass


class InitError_Init_PathRegistryNotFound(InitError):
    pass


class InitError_Init_NoConfigPath(InitError):
    pass


class InitError_Init_NoLogPath(InitError):
    pass


class InitError_Init_PathRegistryNotWritable(InitError):
    pass


class InitError_Init_AppInfoInitFailed(InitError):
    pass


class InitError_Init_Retry(InitError):
    pass


class InitError_Init_InitCanceledByUser(InitError):
    pass


class InitError_Init_AnotherAppLaunching(InitError):
    pass


class InitError_Init_SettingsInitFailed(InitError):
    pass


class InitError_Init_ShuttingDown(InitError):
    pass


class InitError_Init_TooManyObjects(InitError):
    pass


class InitError_Init_NoServerForBackgroundApp(InitError):
    pass


class InitError_Init_NotSupportedWithCompositor(InitError):
    pass


class InitError_Init_NotAvailableToUtilityApps(InitError):
    pass


class InitError_Init_Internal(InitError):
    pass


class InitError_Init_HmdDriverIdIsNone(InitError):
    pass


class InitError_Init_HmdNotFoundPresenceFailed(InitError):
    pass


class InitError_Init_VRMonitorNotFound(InitError):
    pass


class InitError_Init_VRMonitorStartupFailed(InitError):
    pass


class InitError_Init_LowPowerWatchdogNotSupported(InitError):
    pass


class InitError_Init_InvalidApplicationType(InitError):
    pass


class InitError_Init_NotAvailableToWatchdogApps(InitError):
    pass


class InitError_Init_WatchdogDisabledInSettings(InitError):
    pass


class InitError_Init_VRDashboardNotFound(InitError):
    pass


class InitError_Init_VRDashboardStartupFailed(InitError):
    pass


class InitError_Init_VRHomeNotFound(InitError):
    pass


class InitError_Init_VRHomeStartupFailed(InitError):
    pass


class InitError_Init_RebootingBusy(InitError):
    pass


class InitError_Init_FirmwareUpdateBusy(InitError):
    pass


class InitError_Init_FirmwareRecoveryBusy(InitError):
    pass


class InitError_Init_USBServiceBusy(InitError):
    pass


class InitError_Init_VRWebHelperStartupFailed(InitError):
    pass


class InitError_Init_TrackerManagerInitFailed(InitError):
    pass


class InitError_Init_AlreadyRunning(InitError):
    pass


class InitError_Init_FailedForVrMonitor(InitError):
    pass


class InitError_Init_PropertyManagerInitFailed(InitError):
    pass


class InitError_Init_WebServerFailed(InitError):
    pass


class InitError_Init_IllegalTypeTransition(InitError):
    pass


class InitError_Init_MismatchedRuntimes(InitError):
    pass


class InitError_Init_InvalidProcessId(InitError):
    pass


class InitError_Init_VRServiceStartupFailed(InitError):
    pass


class InitError_Init_PrismNeedsNewDrivers(InitError):
    pass


class InitError_Init_PrismStartupTimedOut(InitError):
    pass


class InitError_Init_CouldNotStartPrism(InitError):
    pass


class InitError_Init_CreateDriverDirectDeviceFailed(InitError):
    pass


class InitError_Init_PrismExitedUnexpectedly(InitError):
    pass


class InitError_Driver_Failed(InitError):
    pass


class InitError_Driver_Unknown(InitError):
    pass


class InitError_Driver_HmdUnknown(InitError):
    pass


class InitError_Driver_NotLoaded(InitError):
    pass


class InitError_Driver_RuntimeOutOfDate(InitError):
    pass


class InitError_Driver_HmdInUse(InitError):
    pass


class InitError_Driver_NotCalibrated(InitError):
    pass


class InitError_Driver_CalibrationInvalid(InitError):
    pass


class InitError_Driver_HmdDisplayNotFound(InitError):
    pass


class InitError_Driver_TrackedDeviceInterfaceUnknown(InitError):
    pass


class InitError_Driver_HmdDriverIdOutOfBounds(InitError):
    pass


class InitError_Driver_HmdDisplayMirrored(InitError):
    pass


class InitError_Driver_HmdDisplayNotFoundLaptop(InitError):
    pass


class InitError_IPC_ServerInitFailed(InitError):
    pass


class InitError_IPC_ConnectFailed(InitError):
    pass


class InitError_IPC_SharedStateInitFailed(InitError):
    pass


class InitError_IPC_CompositorInitFailed(InitError):
    pass


class InitError_IPC_MutexInitFailed(InitError):
    pass


class InitError_IPC_Failed(InitError):
    pass


class InitError_IPC_CompositorConnectFailed(InitError):
    pass


class InitError_IPC_CompositorInvalidConnectResponse(InitError):
    pass


class InitError_IPC_ConnectFailedAfterMultipleAttempts(InitError):
    pass


class InitError_IPC_ConnectFailedAfterTargetExited(InitError):
    pass


class InitError_IPC_NamespaceUnavailable(InitError):
    pass


class InitError_Compositor_Failed(InitError):
    pass


class InitError_Compositor_D3D11HardwareRequired(InitError):
    pass


class InitError_Compositor_FirmwareRequiresUpdate(InitError):
    pass


class InitError_Compositor_OverlayInitFailed(InitError):
    pass


class InitError_Compositor_ScreenshotsInitFailed(InitError):
    pass


class InitError_Compositor_UnableToCreateDevice(InitError):
    pass


class InitError_Compositor_SharedStateIsNull(InitError):
    pass


class InitError_Compositor_NotificationManagerIsNull(InitError):
    pass


class InitError_Compositor_ResourceManagerClientIsNull(InitError):
    pass


class InitError_Compositor_MessageOverlaySharedStateInitFailure(InitError):
    pass


class InitError_Compositor_PropertiesInterfaceIsNull(InitError):
    pass


class InitError_Compositor_CreateFullscreenWindowFailed(InitError):
    pass


class InitError_Compositor_SettingsInterfaceIsNull(InitError):
    pass


class InitError_Compositor_FailedToShowWindow(InitError):
    pass


class InitError_Compositor_DistortInterfaceIsNull(InitError):
    pass


class InitError_Compositor_DisplayFrequencyFailure(InitError):
    pass


class InitError_Compositor_RendererInitializationFailed(InitError):
    pass


class InitError_Compositor_DXGIFactoryInterfaceIsNull(InitError):
    pass


class InitError_Compositor_DXGIFactoryCreateFailed(InitError):
    pass


class InitError_Compositor_DXGIFactoryQueryFailed(InitError):
    pass


class InitError_Compositor_InvalidAdapterDesktop(InitError):
    pass


class InitError_Compositor_InvalidHmdAttachment(InitError):
    pass


class InitError_Compositor_InvalidOutputDesktop(InitError):
    pass


class InitError_Compositor_InvalidDeviceProvided(InitError):
    pass


class InitError_Compositor_D3D11RendererInitializationFailed(InitError):
    pass


class InitError_Compositor_FailedToFindDisplayMode(InitError):
    pass


class InitError_Compositor_FailedToCreateSwapChain(InitError):
    pass


class InitError_Compositor_FailedToGetBackBuffer(InitError):
    pass


class InitError_Compositor_FailedToCreateRenderTarget(InitError):
    pass


class InitError_Compositor_FailedToCreateDXGI2SwapChain(InitError):
    pass


class InitError_Compositor_FailedtoGetDXGI2BackBuffer(InitError):
    pass


class InitError_Compositor_FailedToCreateDXGI2RenderTarget(InitError):
    pass


class InitError_Compositor_FailedToGetDXGIDeviceInterface(InitError):
    pass


class InitError_Compositor_SelectDisplayMode(InitError):
    pass


class InitError_Compositor_FailedToCreateNvAPIRenderTargets(InitError):
    pass


class InitError_Compositor_NvAPISetDisplayMode(InitError):
    pass


class InitError_Compositor_FailedToCreateDirectModeDisplay(InitError):
    pass


class InitError_Compositor_InvalidHmdPropertyContainer(InitError):
    pass


class InitError_Compositor_UpdateDisplayFrequency(InitError):
    pass


class InitError_Compositor_CreateRasterizerState(InitError):
    pass


class InitError_Compositor_CreateWireframeRasterizerState(InitError):
    pass


class InitError_Compositor_CreateSamplerState(InitError):
    pass


class InitError_Compositor_CreateClampToBorderSamplerState(InitError):
    pass


class InitError_Compositor_CreateAnisoSamplerState(InitError):
    pass


class InitError_Compositor_CreateOverlaySamplerState(InitError):
    pass


class InitError_Compositor_CreatePanoramaSamplerState(InitError):
    pass


class InitError_Compositor_CreateFontSamplerState(InitError):
    pass


class InitError_Compositor_CreateNoBlendState(InitError):
    pass


class InitError_Compositor_CreateBlendState(InitError):
    pass


class InitError_Compositor_CreateAlphaBlendState(InitError):
    pass


class InitError_Compositor_CreateBlendStateMaskR(InitError):
    pass


class InitError_Compositor_CreateBlendStateMaskG(InitError):
    pass


class InitError_Compositor_CreateBlendStateMaskB(InitError):
    pass


class InitError_Compositor_CreateDepthStencilState(InitError):
    pass


class InitError_Compositor_CreateDepthStencilStateNoWrite(InitError):
    pass


class InitError_Compositor_CreateDepthStencilStateNoDepth(InitError):
    pass


class InitError_Compositor_CreateFlushTexture(InitError):
    pass


class InitError_Compositor_CreateDistortionSurfaces(InitError):
    pass


class InitError_Compositor_CreateConstantBuffer(InitError):
    pass


class InitError_Compositor_CreateHmdPoseConstantBuffer(InitError):
    pass


class InitError_Compositor_CreateHmdPoseStagingConstantBuffer(InitError):
    pass


class InitError_Compositor_CreateSharedFrameInfoConstantBuffer(InitError):
    pass


class InitError_Compositor_CreateOverlayConstantBuffer(InitError):
    pass


class InitError_Compositor_CreateSceneTextureIndexConstantBuffer(InitError):
    pass


class InitError_Compositor_CreateReadableSceneTextureIndexConstantBuffer(InitError):
    pass


class InitError_Compositor_CreateLayerGraphicsTextureIndexConstantBuffer(InitError):
    pass


class InitError_Compositor_CreateLayerComputeTextureIndexConstantBuffer(InitError):
    pass


class InitError_Compositor_CreateLayerComputeSceneTextureIndexConstantBuffer(InitError):
    pass


class InitError_Compositor_CreateComputeHmdPoseConstantBuffer(InitError):
    pass


class InitError_Compositor_CreateGeomConstantBuffer(InitError):
    pass


class InitError_Compositor_CreatePanelMaskConstantBuffer(InitError):
    pass


class InitError_Compositor_CreatePixelSimUBO(InitError):
    pass


class InitError_Compositor_CreateMSAARenderTextures(InitError):
    pass


class InitError_Compositor_CreateResolveRenderTextures(InitError):
    pass


class InitError_Compositor_CreateComputeResolveRenderTextures(InitError):
    pass


class InitError_Compositor_CreateDriverDirectModeResolveTextures(InitError):
    pass


class InitError_Compositor_OpenDriverDirectModeResolveTextures(InitError):
    pass


class InitError_Compositor_CreateFallbackSyncTexture(InitError):
    pass


class InitError_Compositor_ShareFallbackSyncTexture(InitError):
    pass


class InitError_Compositor_CreateOverlayIndexBuffer(InitError):
    pass


class InitError_Compositor_CreateOverlayVertexBuffer(InitError):
    pass


class InitError_Compositor_CreateTextVertexBuffer(InitError):
    pass


class InitError_Compositor_CreateTextIndexBuffer(InitError):
    pass


class InitError_Compositor_CreateMirrorTextures(InitError):
    pass


class InitError_Compositor_CreateLastFrameRenderTexture(InitError):
    pass


class InitError_Compositor_CreateMirrorOverlay(InitError):
    pass


class InitError_Compositor_FailedToCreateVirtualDisplayBackbuffer(InitError):
    pass


class InitError_Compositor_DisplayModeNotSupported(InitError):
    pass


class InitError_Compositor_CreateOverlayInvalidCall(InitError):
    pass


class InitError_Compositor_CreateOverlayAlreadyInitialized(InitError):
    pass


class InitError_Compositor_FailedToCreateMailbox(InitError):
    pass


class InitError_Compositor_WindowInterfaceIsNull(InitError):
    pass


class InitError_Compositor_SystemLayerCreateInstance(InitError):
    pass


class InitError_Compositor_SystemLayerCreateSession(InitError):
    pass


class InitError_VendorSpecific_UnableToConnectToOculusRuntime(InitError):
    pass


class InitError_VendorSpecific_WindowsNotInDevMode(InitError):
    pass


class InitError_VendorSpecific_HmdFound_CantOpenDevice(InitError):
    pass


class InitError_VendorSpecific_HmdFound_UnableToRequestConfigStart(InitError):
    pass


class InitError_VendorSpecific_HmdFound_NoStoredConfig(InitError):
    pass


class InitError_VendorSpecific_HmdFound_ConfigTooBig(InitError):
    pass


class InitError_VendorSpecific_HmdFound_ConfigTooSmall(InitError):
    pass


class InitError_VendorSpecific_HmdFound_UnableToInitZLib(InitError):
    pass


class InitError_VendorSpecific_HmdFound_CantReadFirmwareVersion(InitError):
    pass


class InitError_VendorSpecific_HmdFound_UnableToSendUserDataStart(InitError):
    pass


class InitError_VendorSpecific_HmdFound_UnableToGetUserDataStart(InitError):
    pass


class InitError_VendorSpecific_HmdFound_UnableToGetUserDataNext(InitError):
    pass


class InitError_VendorSpecific_HmdFound_UserDataAddressRange(InitError):
    pass


class InitError_VendorSpecific_HmdFound_UserDataError(InitError):
    pass


class InitError_VendorSpecific_HmdFound_ConfigFailedSanityCheck(InitError):
    pass


class InitError_VendorSpecific_OculusRuntimeBadInstall(InitError):
    pass


class InitError_Steam_SteamInstallationNotFound(InitError):
    pass


class InitError_LastError(InitError):
    pass


InitError.error_index[0] = InitError_None
InitError.error_index[1] = InitError_Unknown
InitError.error_index[100] = InitError_Init_InstallationNotFound
InitError.error_index[101] = InitError_Init_InstallationCorrupt
InitError.error_index[102] = InitError_Init_VRClientDLLNotFound
InitError.error_index[103] = InitError_Init_FileNotFound
InitError.error_index[104] = InitError_Init_FactoryNotFound
InitError.error_index[105] = InitError_Init_InterfaceNotFound
InitError.error_index[106] = InitError_Init_InvalidInterface
InitError.error_index[107] = InitError_Init_UserConfigDirectoryInvalid
InitError.error_index[108] = InitError_Init_HmdNotFound
InitError.error_index[109] = InitError_Init_NotInitialized
InitError.error_index[110] = InitError_Init_PathRegistryNotFound
InitError.error_index[111] = InitError_Init_NoConfigPath
InitError.error_index[112] = InitError_Init_NoLogPath
InitError.error_index[113] = InitError_Init_PathRegistryNotWritable
InitError.error_index[114] = InitError_Init_AppInfoInitFailed
InitError.error_index[115] = InitError_Init_Retry
InitError.error_index[116] = InitError_Init_InitCanceledByUser
InitError.error_index[117] = InitError_Init_AnotherAppLaunching
InitError.error_index[118] = InitError_Init_SettingsInitFailed
InitError.error_index[119] = InitError_Init_ShuttingDown
InitError.error_index[120] = InitError_Init_TooManyObjects
InitError.error_index[121] = InitError_Init_NoServerForBackgroundApp
InitError.error_index[122] = InitError_Init_NotSupportedWithCompositor
InitError.error_index[123] = InitError_Init_NotAvailableToUtilityApps
InitError.error_index[124] = InitError_Init_Internal
InitError.error_index[125] = InitError_Init_HmdDriverIdIsNone
InitError.error_index[126] = InitError_Init_HmdNotFoundPresenceFailed
InitError.error_index[127] = InitError_Init_VRMonitorNotFound
InitError.error_index[128] = InitError_Init_VRMonitorStartupFailed
InitError.error_index[129] = InitError_Init_LowPowerWatchdogNotSupported
InitError.error_index[130] = InitError_Init_InvalidApplicationType
InitError.error_index[131] = InitError_Init_NotAvailableToWatchdogApps
InitError.error_index[132] = InitError_Init_WatchdogDisabledInSettings
InitError.error_index[133] = InitError_Init_VRDashboardNotFound
InitError.error_index[134] = InitError_Init_VRDashboardStartupFailed
InitError.error_index[135] = InitError_Init_VRHomeNotFound
InitError.error_index[136] = InitError_Init_VRHomeStartupFailed
InitError.error_index[137] = InitError_Init_RebootingBusy
InitError.error_index[138] = InitError_Init_FirmwareUpdateBusy
InitError.error_index[139] = InitError_Init_FirmwareRecoveryBusy
InitError.error_index[140] = InitError_Init_USBServiceBusy
InitError.error_index[141] = InitError_Init_VRWebHelperStartupFailed
InitError.error_index[142] = InitError_Init_TrackerManagerInitFailed
InitError.error_index[143] = InitError_Init_AlreadyRunning
InitError.error_index[144] = InitError_Init_FailedForVrMonitor
InitError.error_index[145] = InitError_Init_PropertyManagerInitFailed
InitError.error_index[146] = InitError_Init_WebServerFailed
InitError.error_index[147] = InitError_Init_IllegalTypeTransition
InitError.error_index[148] = InitError_Init_MismatchedRuntimes
InitError.error_index[149] = InitError_Init_InvalidProcessId
InitError.error_index[150] = InitError_Init_VRServiceStartupFailed
InitError.error_index[151] = InitError_Init_PrismNeedsNewDrivers
InitError.error_index[152] = InitError_Init_PrismStartupTimedOut
InitError.error_index[153] = InitError_Init_CouldNotStartPrism
InitError.error_index[154] = InitError_Init_CreateDriverDirectDeviceFailed
InitError.error_index[155] = InitError_Init_PrismExitedUnexpectedly
InitError.error_index[200] = InitError_Driver_Failed
InitError.error_index[201] = InitError_Driver_Unknown
InitError.error_index[202] = InitError_Driver_HmdUnknown
InitError.error_index[203] = InitError_Driver_NotLoaded
InitError.error_index[204] = InitError_Driver_RuntimeOutOfDate
InitError.error_index[205] = InitError_Driver_HmdInUse
InitError.error_index[206] = InitError_Driver_NotCalibrated
InitError.error_index[207] = InitError_Driver_CalibrationInvalid
InitError.error_index[208] = InitError_Driver_HmdDisplayNotFound
InitError.error_index[209] = InitError_Driver_TrackedDeviceInterfaceUnknown
InitError.error_index[211] = InitError_Driver_HmdDriverIdOutOfBounds
InitError.error_index[212] = InitError_Driver_HmdDisplayMirrored
InitError.error_index[213] = InitError_Driver_HmdDisplayNotFoundLaptop
InitError.error_index[300] = InitError_IPC_ServerInitFailed
InitError.error_index[301] = InitError_IPC_ConnectFailed
InitError.error_index[302] = InitError_IPC_SharedStateInitFailed
InitError.error_index[303] = InitError_IPC_CompositorInitFailed
InitError.error_index[304] = InitError_IPC_MutexInitFailed
InitError.error_index[305] = InitError_IPC_Failed
InitError.error_index[306] = InitError_IPC_CompositorConnectFailed
InitError.error_index[307] = InitError_IPC_CompositorInvalidConnectResponse
InitError.error_index[308] = InitError_IPC_ConnectFailedAfterMultipleAttempts
InitError.error_index[309] = InitError_IPC_ConnectFailedAfterTargetExited
InitError.error_index[310] = InitError_IPC_NamespaceUnavailable
InitError.error_index[400] = InitError_Compositor_Failed
InitError.error_index[401] = InitError_Compositor_D3D11HardwareRequired
InitError.error_index[402] = InitError_Compositor_FirmwareRequiresUpdate
InitError.error_index[403] = InitError_Compositor_OverlayInitFailed
InitError.error_index[404] = InitError_Compositor_ScreenshotsInitFailed
InitError.error_index[405] = InitError_Compositor_UnableToCreateDevice
InitError.error_index[406] = InitError_Compositor_SharedStateIsNull
InitError.error_index[407] = InitError_Compositor_NotificationManagerIsNull
InitError.error_index[408] = InitError_Compositor_ResourceManagerClientIsNull
InitError.error_index[409] = InitError_Compositor_MessageOverlaySharedStateInitFailure
InitError.error_index[410] = InitError_Compositor_PropertiesInterfaceIsNull
InitError.error_index[411] = InitError_Compositor_CreateFullscreenWindowFailed
InitError.error_index[412] = InitError_Compositor_SettingsInterfaceIsNull
InitError.error_index[413] = InitError_Compositor_FailedToShowWindow
InitError.error_index[414] = InitError_Compositor_DistortInterfaceIsNull
InitError.error_index[415] = InitError_Compositor_DisplayFrequencyFailure
InitError.error_index[416] = InitError_Compositor_RendererInitializationFailed
InitError.error_index[417] = InitError_Compositor_DXGIFactoryInterfaceIsNull
InitError.error_index[418] = InitError_Compositor_DXGIFactoryCreateFailed
InitError.error_index[419] = InitError_Compositor_DXGIFactoryQueryFailed
InitError.error_index[420] = InitError_Compositor_InvalidAdapterDesktop
InitError.error_index[421] = InitError_Compositor_InvalidHmdAttachment
InitError.error_index[422] = InitError_Compositor_InvalidOutputDesktop
InitError.error_index[423] = InitError_Compositor_InvalidDeviceProvided
InitError.error_index[424] = InitError_Compositor_D3D11RendererInitializationFailed
InitError.error_index[425] = InitError_Compositor_FailedToFindDisplayMode
InitError.error_index[426] = InitError_Compositor_FailedToCreateSwapChain
InitError.error_index[427] = InitError_Compositor_FailedToGetBackBuffer
InitError.error_index[428] = InitError_Compositor_FailedToCreateRenderTarget
InitError.error_index[429] = InitError_Compositor_FailedToCreateDXGI2SwapChain
InitError.error_index[430] = InitError_Compositor_FailedtoGetDXGI2BackBuffer
InitError.error_index[431] = InitError_Compositor_FailedToCreateDXGI2RenderTarget
InitError.error_index[432] = InitError_Compositor_FailedToGetDXGIDeviceInterface
InitError.error_index[433] = InitError_Compositor_SelectDisplayMode
InitError.error_index[434] = InitError_Compositor_FailedToCreateNvAPIRenderTargets
InitError.error_index[435] = InitError_Compositor_NvAPISetDisplayMode
InitError.error_index[436] = InitError_Compositor_FailedToCreateDirectModeDisplay
InitError.error_index[437] = InitError_Compositor_InvalidHmdPropertyContainer
InitError.error_index[438] = InitError_Compositor_UpdateDisplayFrequency
InitError.error_index[439] = InitError_Compositor_CreateRasterizerState
InitError.error_index[440] = InitError_Compositor_CreateWireframeRasterizerState
InitError.error_index[441] = InitError_Compositor_CreateSamplerState
InitError.error_index[442] = InitError_Compositor_CreateClampToBorderSamplerState
InitError.error_index[443] = InitError_Compositor_CreateAnisoSamplerState
InitError.error_index[444] = InitError_Compositor_CreateOverlaySamplerState
InitError.error_index[445] = InitError_Compositor_CreatePanoramaSamplerState
InitError.error_index[446] = InitError_Compositor_CreateFontSamplerState
InitError.error_index[447] = InitError_Compositor_CreateNoBlendState
InitError.error_index[448] = InitError_Compositor_CreateBlendState
InitError.error_index[449] = InitError_Compositor_CreateAlphaBlendState
InitError.error_index[450] = InitError_Compositor_CreateBlendStateMaskR
InitError.error_index[451] = InitError_Compositor_CreateBlendStateMaskG
InitError.error_index[452] = InitError_Compositor_CreateBlendStateMaskB
InitError.error_index[453] = InitError_Compositor_CreateDepthStencilState
InitError.error_index[454] = InitError_Compositor_CreateDepthStencilStateNoWrite
InitError.error_index[455] = InitError_Compositor_CreateDepthStencilStateNoDepth
InitError.error_index[456] = InitError_Compositor_CreateFlushTexture
InitError.error_index[457] = InitError_Compositor_CreateDistortionSurfaces
InitError.error_index[458] = InitError_Compositor_CreateConstantBuffer
InitError.error_index[459] = InitError_Compositor_CreateHmdPoseConstantBuffer
InitError.error_index[460] = InitError_Compositor_CreateHmdPoseStagingConstantBuffer
InitError.error_index[461] = InitError_Compositor_CreateSharedFrameInfoConstantBuffer
InitError.error_index[462] = InitError_Compositor_CreateOverlayConstantBuffer
InitError.error_index[463] = InitError_Compositor_CreateSceneTextureIndexConstantBuffer
InitError.error_index[464] = InitError_Compositor_CreateReadableSceneTextureIndexConstantBuffer
InitError.error_index[465] = InitError_Compositor_CreateLayerGraphicsTextureIndexConstantBuffer
InitError.error_index[466] = InitError_Compositor_CreateLayerComputeTextureIndexConstantBuffer
InitError.error_index[467] = InitError_Compositor_CreateLayerComputeSceneTextureIndexConstantBuffer
InitError.error_index[468] = InitError_Compositor_CreateComputeHmdPoseConstantBuffer
InitError.error_index[469] = InitError_Compositor_CreateGeomConstantBuffer
InitError.error_index[470] = InitError_Compositor_CreatePanelMaskConstantBuffer
InitError.error_index[471] = InitError_Compositor_CreatePixelSimUBO
InitError.error_index[472] = InitError_Compositor_CreateMSAARenderTextures
InitError.error_index[473] = InitError_Compositor_CreateResolveRenderTextures
InitError.error_index[474] = InitError_Compositor_CreateComputeResolveRenderTextures
InitError.error_index[475] = InitError_Compositor_CreateDriverDirectModeResolveTextures
InitError.error_index[476] = InitError_Compositor_OpenDriverDirectModeResolveTextures
InitError.error_index[477] = InitError_Compositor_CreateFallbackSyncTexture
InitError.error_index[478] = InitError_Compositor_ShareFallbackSyncTexture
InitError.error_index[479] = InitError_Compositor_CreateOverlayIndexBuffer
InitError.error_index[480] = InitError_Compositor_CreateOverlayVertexBuffer
InitError.error_index[481] = InitError_Compositor_CreateTextVertexBuffer
InitError.error_index[482] = InitError_Compositor_CreateTextIndexBuffer
InitError.error_index[483] = InitError_Compositor_CreateMirrorTextures
InitError.error_index[484] = InitError_Compositor_CreateLastFrameRenderTexture
InitError.error_index[485] = InitError_Compositor_CreateMirrorOverlay
InitError.error_index[486] = InitError_Compositor_FailedToCreateVirtualDisplayBackbuffer
InitError.error_index[487] = InitError_Compositor_DisplayModeNotSupported
InitError.error_index[488] = InitError_Compositor_CreateOverlayInvalidCall
InitError.error_index[489] = InitError_Compositor_CreateOverlayAlreadyInitialized
InitError.error_index[490] = InitError_Compositor_FailedToCreateMailbox
InitError.error_index[491] = InitError_Compositor_WindowInterfaceIsNull
InitError.error_index[492] = InitError_Compositor_SystemLayerCreateInstance
InitError.error_index[493] = InitError_Compositor_SystemLayerCreateSession
InitError.error_index[1000] = InitError_VendorSpecific_UnableToConnectToOculusRuntime
InitError.error_index[1001] = InitError_VendorSpecific_WindowsNotInDevMode
InitError.error_index[1101] = InitError_VendorSpecific_HmdFound_CantOpenDevice
InitError.error_index[1102] = InitError_VendorSpecific_HmdFound_UnableToRequestConfigStart
InitError.error_index[1103] = InitError_VendorSpecific_HmdFound_NoStoredConfig
InitError.error_index[1104] = InitError_VendorSpecific_HmdFound_ConfigTooBig
InitError.error_index[1105] = InitError_VendorSpecific_HmdFound_ConfigTooSmall
InitError.error_index[1106] = InitError_VendorSpecific_HmdFound_UnableToInitZLib
InitError.error_index[1107] = InitError_VendorSpecific_HmdFound_CantReadFirmwareVersion
InitError.error_index[1108] = InitError_VendorSpecific_HmdFound_UnableToSendUserDataStart
InitError.error_index[1109] = InitError_VendorSpecific_HmdFound_UnableToGetUserDataStart
InitError.error_index[1110] = InitError_VendorSpecific_HmdFound_UnableToGetUserDataNext
InitError.error_index[1111] = InitError_VendorSpecific_HmdFound_UserDataAddressRange
InitError.error_index[1112] = InitError_VendorSpecific_HmdFound_UserDataError
InitError.error_index[1113] = InitError_VendorSpecific_HmdFound_ConfigFailedSanityCheck
InitError.error_index[1114] = InitError_VendorSpecific_OculusRuntimeBadInstall
InitError.error_index[2000] = InitError_Steam_SteamInstallationNotFound
InitError.error_index[2001] = InitError_LastError


class TrackedCameraError(ErrorCode):
    error_index = dict()


class TrackedCameraError_None(TrackedCameraError):
    is_error = False


class TrackedCameraError_OperationFailed(TrackedCameraError):
    pass


class TrackedCameraError_InvalidHandle(TrackedCameraError):
    pass


class TrackedCameraError_InvalidFrameHeaderVersion(TrackedCameraError):
    pass


class TrackedCameraError_OutOfHandles(TrackedCameraError):
    pass


class TrackedCameraError_IPCFailure(TrackedCameraError):
    pass


class TrackedCameraError_NotSupportedForThisDevice(TrackedCameraError):
    pass


class TrackedCameraError_SharedMemoryFailure(TrackedCameraError):
    pass


class TrackedCameraError_FrameBufferingFailure(TrackedCameraError):
    pass


class TrackedCameraError_StreamSetupFailure(TrackedCameraError):
    pass


class TrackedCameraError_InvalidGLTextureId(TrackedCameraError):
    pass


class TrackedCameraError_InvalidSharedTextureHandle(TrackedCameraError):
    pass


class TrackedCameraError_FailedToGetGLTextureId(TrackedCameraError):
    pass


class TrackedCameraError_SharedTextureFailure(TrackedCameraError):
    pass


class TrackedCameraError_NoFrameAvailable(TrackedCameraError):
    pass


class TrackedCameraError_InvalidArgument(TrackedCameraError):
    pass


class TrackedCameraError_InvalidFrameBufferSize(TrackedCameraError):
    pass


TrackedCameraError.error_index[0] = TrackedCameraError_None
TrackedCameraError.error_index[100] = TrackedCameraError_OperationFailed
TrackedCameraError.error_index[101] = TrackedCameraError_InvalidHandle
TrackedCameraError.error_index[102] = TrackedCameraError_InvalidFrameHeaderVersion
TrackedCameraError.error_index[103] = TrackedCameraError_OutOfHandles
TrackedCameraError.error_index[104] = TrackedCameraError_IPCFailure
TrackedCameraError.error_index[105] = TrackedCameraError_NotSupportedForThisDevice
TrackedCameraError.error_index[106] = TrackedCameraError_SharedMemoryFailure
TrackedCameraError.error_index[107] = TrackedCameraError_FrameBufferingFailure
TrackedCameraError.error_index[108] = TrackedCameraError_StreamSetupFailure
TrackedCameraError.error_index[109] = TrackedCameraError_InvalidGLTextureId
TrackedCameraError.error_index[110] = TrackedCameraError_InvalidSharedTextureHandle
TrackedCameraError.error_index[111] = TrackedCameraError_FailedToGetGLTextureId
TrackedCameraError.error_index[112] = TrackedCameraError_SharedTextureFailure
TrackedCameraError.error_index[113] = TrackedCameraError_NoFrameAvailable
TrackedCameraError.error_index[114] = TrackedCameraError_InvalidArgument
TrackedCameraError.error_index[115] = TrackedCameraError_InvalidFrameBufferSize


class ApplicationError(ErrorCode):
    error_index = dict()


class ApplicationError_None(ApplicationError):
    is_error = False


class ApplicationError_AppKeyAlreadyExists(ApplicationError):
    pass


class ApplicationError_NoManifest(ApplicationError):
    pass


class ApplicationError_NoApplication(ApplicationError):
    pass


class ApplicationError_InvalidIndex(ApplicationError):
    pass


class ApplicationError_UnknownApplication(ApplicationError):
    pass


class ApplicationError_IPCFailed(ApplicationError):
    pass


class ApplicationError_ApplicationAlreadyRunning(ApplicationError):
    pass


class ApplicationError_InvalidManifest(ApplicationError):
    pass


class ApplicationError_InvalidApplication(ApplicationError):
    pass


class ApplicationError_LaunchFailed(ApplicationError):
    pass


class ApplicationError_ApplicationAlreadyStarting(ApplicationError):
    pass


class ApplicationError_LaunchInProgress(ApplicationError):
    pass


class ApplicationError_OldApplicationQuitting(ApplicationError):
    pass


class ApplicationError_TransitionAborted(ApplicationError):
    pass


class ApplicationError_IsTemplate(ApplicationError):
    pass


class ApplicationError_SteamVRIsExiting(ApplicationError):
    pass


class ApplicationError_BufferTooSmall(ApplicationError, BufferTooSmallError):
    pass


class ApplicationError_PropertyNotSet(ApplicationError):
    pass


class ApplicationError_UnknownProperty(ApplicationError):
    pass


class ApplicationError_InvalidParameter(ApplicationError):
    pass


class ApplicationError_NotImplemented(ApplicationError):
    pass


ApplicationError.error_index[0] = ApplicationError_None
ApplicationError.error_index[100] = ApplicationError_AppKeyAlreadyExists
ApplicationError.error_index[101] = ApplicationError_NoManifest
ApplicationError.error_index[102] = ApplicationError_NoApplication
ApplicationError.error_index[103] = ApplicationError_InvalidIndex
ApplicationError.error_index[104] = ApplicationError_UnknownApplication
ApplicationError.error_index[105] = ApplicationError_IPCFailed
ApplicationError.error_index[106] = ApplicationError_ApplicationAlreadyRunning
ApplicationError.error_index[107] = ApplicationError_InvalidManifest
ApplicationError.error_index[108] = ApplicationError_InvalidApplication
ApplicationError.error_index[109] = ApplicationError_LaunchFailed
ApplicationError.error_index[110] = ApplicationError_ApplicationAlreadyStarting
ApplicationError.error_index[111] = ApplicationError_LaunchInProgress
ApplicationError.error_index[112] = ApplicationError_OldApplicationQuitting
ApplicationError.error_index[113] = ApplicationError_TransitionAborted
ApplicationError.error_index[114] = ApplicationError_IsTemplate
ApplicationError.error_index[115] = ApplicationError_SteamVRIsExiting
ApplicationError.error_index[200] = ApplicationError_BufferTooSmall
ApplicationError.error_index[201] = ApplicationError_PropertyNotSet
ApplicationError.error_index[202] = ApplicationError_UnknownProperty
ApplicationError.error_index[203] = ApplicationError_InvalidParameter
ApplicationError.error_index[300] = ApplicationError_NotImplemented


class SettingsError(ErrorCode):
    error_index = dict()


class SettingsError_None(SettingsError):
    is_error = False


class SettingsError_IPCFailed(SettingsError):
    pass


class SettingsError_WriteFailed(SettingsError):
    pass


class SettingsError_ReadFailed(SettingsError):
    pass


class SettingsError_JsonParseFailed(SettingsError):
    pass


class SettingsError_UnsetSettingHasNoDefault(SettingsError):
    pass


SettingsError.error_index[0] = SettingsError_None
SettingsError.error_index[1] = SettingsError_IPCFailed
SettingsError.error_index[2] = SettingsError_WriteFailed
SettingsError.error_index[3] = SettingsError_ReadFailed
SettingsError.error_index[4] = SettingsError_JsonParseFailed
SettingsError.error_index[5] = SettingsError_UnsetSettingHasNoDefault


class CompositorError(ErrorCode):
    error_index = dict()


class CompositorError_None(CompositorError):
    is_error = False


class CompositorError_RequestFailed(CompositorError):
    pass


class CompositorError_IncompatibleVersion(CompositorError):
    pass


class CompositorError_DoNotHaveFocus(CompositorError):
    pass


class CompositorError_InvalidTexture(CompositorError):
    pass


class CompositorError_IsNotSceneApplication(CompositorError):
    pass


class CompositorError_TextureIsOnWrongDevice(CompositorError):
    pass


class CompositorError_TextureUsesUnsupportedFormat(CompositorError):
    pass


class CompositorError_SharedTexturesNotSupported(CompositorError):
    pass


class CompositorError_IndexOutOfRange(CompositorError):
    pass


class CompositorError_AlreadySubmitted(CompositorError):
    pass


class CompositorError_InvalidBounds(CompositorError):
    pass


class CompositorError_AlreadySet(CompositorError):
    pass


CompositorError.error_index[0] = CompositorError_None
CompositorError.error_index[1] = CompositorError_RequestFailed
CompositorError.error_index[100] = CompositorError_IncompatibleVersion
CompositorError.error_index[101] = CompositorError_DoNotHaveFocus
CompositorError.error_index[102] = CompositorError_InvalidTexture
CompositorError.error_index[103] = CompositorError_IsNotSceneApplication
CompositorError.error_index[104] = CompositorError_TextureIsOnWrongDevice
CompositorError.error_index[105] = CompositorError_TextureUsesUnsupportedFormat
CompositorError.error_index[106] = CompositorError_SharedTexturesNotSupported
CompositorError.error_index[107] = CompositorError_IndexOutOfRange
CompositorError.error_index[108] = CompositorError_AlreadySubmitted
CompositorError.error_index[109] = CompositorError_InvalidBounds
CompositorError.error_index[110] = CompositorError_AlreadySet


class RenderModelError(ErrorCode):
    error_index = dict()


class RenderModelError_None(RenderModelError):
    is_error = False


class RenderModelError_Loading(RenderModelError):
    pass


class RenderModelError_NotSupported(RenderModelError):
    pass


class RenderModelError_InvalidArg(RenderModelError):
    pass


class RenderModelError_InvalidModel(RenderModelError):
    pass


class RenderModelError_NoShapes(RenderModelError):
    pass


class RenderModelError_MultipleShapes(RenderModelError):
    pass


class RenderModelError_TooManyVertices(RenderModelError):
    pass


class RenderModelError_MultipleTextures(RenderModelError):
    pass


class RenderModelError_BufferTooSmall(RenderModelError, BufferTooSmallError):
    pass


class RenderModelError_NotEnoughNormals(RenderModelError):
    pass


class RenderModelError_NotEnoughTexCoords(RenderModelError):
    pass


class RenderModelError_InvalidTexture(RenderModelError):
    pass


RenderModelError.error_index[0] = RenderModelError_None
RenderModelError.error_index[100] = RenderModelError_Loading
RenderModelError.error_index[200] = RenderModelError_NotSupported
RenderModelError.error_index[300] = RenderModelError_InvalidArg
RenderModelError.error_index[301] = RenderModelError_InvalidModel
RenderModelError.error_index[302] = RenderModelError_NoShapes
RenderModelError.error_index[303] = RenderModelError_MultipleShapes
RenderModelError.error_index[304] = RenderModelError_TooManyVertices
RenderModelError.error_index[305] = RenderModelError_MultipleTextures
RenderModelError.error_index[306] = RenderModelError_BufferTooSmall
RenderModelError.error_index[307] = RenderModelError_NotEnoughNormals
RenderModelError.error_index[308] = RenderModelError_NotEnoughTexCoords
RenderModelError.error_index[400] = RenderModelError_InvalidTexture


class ScreenshotError(ErrorCode):
    error_index = dict()


class ScreenshotError_None(ScreenshotError):
    is_error = False


class ScreenshotError_RequestFailed(ScreenshotError):
    pass


class ScreenshotError_IncompatibleVersion(ScreenshotError):
    pass


class ScreenshotError_NotFound(ScreenshotError):
    pass


class ScreenshotError_BufferTooSmall(ScreenshotError, BufferTooSmallError):
    pass


class ScreenshotError_ScreenshotAlreadyInProgress(ScreenshotError):
    pass


ScreenshotError.error_index[0] = ScreenshotError_None
ScreenshotError.error_index[1] = ScreenshotError_RequestFailed
ScreenshotError.error_index[100] = ScreenshotError_IncompatibleVersion
ScreenshotError.error_index[101] = ScreenshotError_NotFound
ScreenshotError.error_index[102] = ScreenshotError_BufferTooSmall
ScreenshotError.error_index[108] = ScreenshotError_ScreenshotAlreadyInProgress


class IOBufferError(ErrorCode):
    error_index = dict()


class IOBuffer_Success(IOBufferError):
    is_error = False


class IOBuffer_OperationFailed(IOBufferError):
    pass


class IOBuffer_InvalidHandle(IOBufferError):
    pass


class IOBuffer_InvalidArgument(IOBufferError):
    pass


class IOBuffer_PathExists(IOBufferError):
    pass


class IOBuffer_PathDoesNotExist(IOBufferError):
    pass


class IOBuffer_Permission(IOBufferError):
    pass


IOBufferError.error_index[0] = IOBuffer_Success
IOBufferError.error_index[100] = IOBuffer_OperationFailed
IOBufferError.error_index[101] = IOBuffer_InvalidHandle
IOBufferError.error_index[102] = IOBuffer_InvalidArgument
IOBufferError.error_index[103] = IOBuffer_PathExists
IOBufferError.error_index[104] = IOBuffer_PathDoesNotExist
IOBufferError.error_index[105] = IOBuffer_Permission


class DebugError(ErrorCode):
    error_index = dict()


class DebugError_Success(DebugError):
    is_error = False


class DebugError_BadParameter(DebugError):
    pass


DebugError.error_index[0] = DebugError_Success
DebugError.error_index[1] = DebugError_BadParameter



#!/bin/env python

# file openvr_gl_renderer.py

from OpenGL.GL import *  # @UnusedWildImport # this comment squelches an IDE warning
import numpy

import openvr

"""
Renders OpenGL scenes to virtual reality headsets using OpenVR API
"""


# TODO: matrixForOpenVrMatrix() is not general, it is specific the perspective and 
# modelview matrices used in this example
def matrixForOpenVrMatrix(mat):
    if len(mat.m) == 4: # HmdMatrix44_t?
        result = numpy.matrix(
                ((mat.m[0][0], mat.m[1][0], mat.m[2][0], mat.m[3][0]),
                 (mat.m[0][1], mat.m[1][1], mat.m[2][1], mat.m[3][1]), 
                 (mat.m[0][2], mat.m[1][2], mat.m[2][2], mat.m[3][2]), 
                 (mat.m[0][3], mat.m[1][3], mat.m[2][3], mat.m[3][3]),)
            , numpy.float32)
    elif len(mat.m) == 3: # HmdMatrix34_t?
        result = numpy.matrix(
                ((mat.m[0][0], mat.m[1][0], mat.m[2][0], 0.0),
                 (mat.m[0][1], mat.m[1][1], mat.m[2][1], 0.0), 
                 (mat.m[0][2], mat.m[1][2], mat.m[2][2], 0.0), 
                 (mat.m[0][3], mat.m[1][3], mat.m[2][3], 1.0),)
            , numpy.float32)
    return result


class OpenVrFramebuffer(object):
    "Framebuffer for rendering one eye"
    
    def __init__(self, width, height, multisample = 0):
        self.fb = 0
        self.depth_buffer = 0
        self.texture_id = 0
        self.width = width
        self.height = height
        self.compositor = None
        self.multisample = multisample
        
    def init_gl(self):
        # Set up framebuffer and render textures
        self.fb = glGenFramebuffers(1)
        glBindFramebuffer(GL_FRAMEBUFFER, self.fb)
        self.depth_buffer = glGenRenderbuffers(1)
        glBindRenderbuffer(GL_RENDERBUFFER, self.depth_buffer)
        if self.multisample > 0:
            glRenderbufferStorageMultisample(GL_RENDERBUFFER, self.multisample, GL_DEPTH24_STENCIL8, self.width, self.height)
        else:
            glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH24_STENCIL8, self.width, self.height)
        glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_STENCIL_ATTACHMENT, GL_RENDERBUFFER, self.depth_buffer)
        self.texture_id = int(glGenTextures(1))
        if self.multisample > 0:
            glBindTexture(GL_TEXTURE_2D_MULTISAMPLE, self.texture_id)
            glTexImage2DMultisample(GL_TEXTURE_2D_MULTISAMPLE, self.multisample, GL_RGBA8, self.width, self.height, True)
            glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D_MULTISAMPLE, self.texture_id, 0)
        else:
            glBindTexture(GL_TEXTURE_2D, self.texture_id)
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, self.width, self.height, 0, GL_RGBA, GL_UNSIGNED_BYTE, None)
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0)
            glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, self.texture_id, 0)
        status = glCheckFramebufferStatus(GL_FRAMEBUFFER)
        if status != GL_FRAMEBUFFER_COMPLETE:
            glBindFramebuffer(GL_FRAMEBUFFER, 0)
            raise Exception("Incomplete framebuffer")
        # Resolver framebuffer in case of multisample antialiasing
        if self.multisample > 0:
            self.resolve_fb = glGenFramebuffers(1)
            glBindFramebuffer(GL_FRAMEBUFFER, self.resolve_fb)
            self.resolve_texture_id = int(glGenTextures(1))
            glBindTexture(GL_TEXTURE_2D, self.resolve_texture_id)
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0)
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, self.width, self.height, 0, GL_RGBA, GL_UNSIGNED_BYTE, None)
            glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, self.resolve_texture_id, 0)
            status = glCheckFramebufferStatus(GL_FRAMEBUFFER)
            if status != GL_FRAMEBUFFER_COMPLETE:
                glBindFramebuffer(GL_FRAMEBUFFER, 0)
                raise Exception("Incomplete framebuffer")
        glBindFramebuffer(GL_FRAMEBUFFER, 0)   
        # OpenVR texture data
        self.texture = openvr.Texture_t()
        if self.multisample > 0:
            self.texture.handle = self.resolve_texture_id
        else:
            self.texture.handle = self.texture_id
        self.texture.eType = openvr.TextureType_OpenGL
        self.texture.eColorSpace = openvr.ColorSpace_Gamma
        
    def submit(self, eye):
        if self.multisample > 0:
            glBindFramebuffer(GL_READ_FRAMEBUFFER, self.fb)
            glBindFramebuffer(GL_DRAW_FRAMEBUFFER, self.resolve_fb)
            glBlitFramebuffer(0, 0, self.width, self.height, 
                              0, 0, self.width, self.height,
                              GL_COLOR_BUFFER_BIT, GL_LINEAR)
            glBindFramebuffer(GL_READ_FRAMEBUFFER, 0)
            glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0)
        openvr.VRCompositor().submit(eye, self.texture)
        
    def dispose_gl(self):
        glDeleteTextures([self.texture_id])
        glDeleteRenderbuffers(1, [self.depth_buffer])
        glDeleteFramebuffers(1, [self.fb])
        self.fb = 0
        if self.multisample > 0:
            glDeleteTextures([self.resolve_texture_id])
            glDeleteFramebuffers(1, [self.resolve_fb])


class OpenVrGlRenderer(list):
    "Renders to virtual reality headset using OpenVR and OpenGL APIs"

    def __init__(self, actor=None, window_size=(800,600), multisample=0):
        self.vr_system = None
        self.left_fb = None
        self.right_fb = None
        self.window_size = window_size
        poses_t = openvr.TrackedDevicePose_t * openvr.k_unMaxTrackedDeviceCount
        self.poses = poses_t()
        if actor is not None:
            try:
                len(actor)
                self.extend(actor)
            except TypeError:
                self.append(actor)
        self.do_mirror = False
        self.multisample = multisample
        self.compositor = None

    def init_gl(self):
        "allocate OpenGL resources"
        self.vr_system = openvr.init(openvr.VRApplication_Scene)
        w, h = self.vr_system.getRecommendedRenderTargetSize()
        self.left_fb = OpenVrFramebuffer(w, h, multisample=self.multisample)
        self.right_fb = OpenVrFramebuffer(w, h, multisample=self.multisample)
        self.compositor = openvr.VRCompositor()
        if self.compositor is None:
            raise Exception("Unable to create compositor") 
        self.left_fb.init_gl()
        self.right_fb.init_gl()
        # Compute projection matrix
        zNear = 0.2
        zFar = 500.0
        self.projection_left = numpy.asarray(matrixForOpenVrMatrix(self.vr_system.getProjectionMatrix(
                openvr.Eye_Left, 
                zNear, zFar)))
        self.projection_right = numpy.asarray(matrixForOpenVrMatrix(self.vr_system.getProjectionMatrix(
                openvr.Eye_Right, 
                zNear, zFar)))
        self.view_left = matrixForOpenVrMatrix(
            self.vr_system.getEyeToHeadTransform(openvr.Eye_Left)).I  # head_X_eye in Kane notation
        self.view_right = matrixForOpenVrMatrix(
            self.vr_system.getEyeToHeadTransform(openvr.Eye_Right)).I  # head_X_eye in Kane notation
        for actor in self:
            actor.init_gl()

    def render_scene(self):
        if self.compositor is None:
            return
        self.compositor.waitGetPoses(self.poses, None)
        hmd_pose0 = self.poses[openvr.k_unTrackedDeviceIndex_Hmd]
        if not hmd_pose0.bPoseIsValid:
            return
        hmd_pose1 = hmd_pose0.mDeviceToAbsoluteTracking # head_X_room in Kane notation
        hmd_pose = matrixForOpenVrMatrix(hmd_pose1).I # room_X_head in Kane notation
        # Use the pose to compute things
        modelview = hmd_pose
        mvl = modelview * self.view_left # room_X_eye(left) in Kane notation
        mvr = modelview * self.view_right # room_X_eye(right) in Kane notation
        # Repack the resulting matrices to have default stride, to avoid
        # problems with weird strides and OpenGL
        mvl = numpy.asarray(numpy.matrix(mvl, dtype=numpy.float32))
        mvr = numpy.asarray(numpy.matrix(mvr, dtype=numpy.float32))
        # 1) On-screen render:
        if self.do_mirror:
            glViewport(0, 0, self.window_size[0], self.window_size[1])
            # Display left eye view to screen
            self.display_gl(mvl, self.projection_left)
        # 2) VR render
        # Left eye view
        glBindFramebuffer(GL_FRAMEBUFFER, self.left_fb.fb)
        glViewport(0, 0, self.left_fb.width, self.left_fb.height)
        self.display_gl(mvl, self.projection_left)
        self.left_fb.submit(openvr.Eye_Left)
        # self.compositor.submit(openvr.Eye_Left, self.left_fb.texture)
        # Right eye view
        glBindFramebuffer(GL_FRAMEBUFFER, self.right_fb.fb)
        self.display_gl(mvr, self.projection_right)
        self.right_fb.submit(openvr.Eye_Right)
        # self.compositor.submit(openvr.Eye_Right, self.right_fb.texture)
        glBindFramebuffer(GL_FRAMEBUFFER, 0)
        
    def display_gl(self, modelview, projection):
        glClearColor(0.5, 0.5, 0.5, 0.0) # gray background
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        for actor in self:
            actor.display_gl(modelview, projection)

    def dispose_gl(self):
        for actor in self:
            actor.dispose_gl()
        if self.vr_system is not None:
            openvr.shutdown()
            self.vr_system = None
        if self.left_fb is not None:
            self.left_fb.dispose_gl()
            self.right_fb.dispose_gl()



#!/bin/env python

# file hello_wx.py

import sys

import wx
from wx import glcanvas


"""
Minimal wxPython programming example which colored OpenGL cube scene that can be closed by pressing ESCAPE.
"""

class WxApp(wx.App):
	"HelloApp uses wxPython library to create an opengl context, listen to keyboard events, and clean up"
	renderer = None
	title = None
	_is_initialized = False
	window = None
	canvas = None
	context = None
	running = False

	def __init__(self, renderer, title="wx test"):
		"Creates an OpenGL context and a window, and acquires OpenGL resources"
		self.renderer = renderer
		self.title = title
		self._is_initialized = False # keep track of whether self.init_gl() has been called
		self.window = None
		
		wx.App.__init__(self, redirect = False)
	
	def OnInit ( self ):
		self.init_gl()
		return True
	
	def OnIdle ( self, evt ):
		self.window.Refresh(False)
		evt.RequestMore()
	
	def __enter__(self):
		"setup for RAII using 'with' keyword"
		return self
	
	def __exit__(self, type_arg, value, traceback):
		"cleanup for RAII using 'with' keyword"
		self.dispose_gl()
	
	def init_gl(self):
		print('creating Frame')
		self.window = wx.Frame ( parent=None, id=wx.ID_ANY, title=self.title,
			style=wx.DEFAULT_FRAME_STYLE|wx.WS_EX_PROCESS_IDLE )
		print('creating GLCanvas')
		self.canvas = glcanvas.GLCanvas ( self.window, glcanvas.WX_GL_RGBA )
		print('creating GLContext')
		self.context = glcanvas.GLContext(self.canvas)
		self.canvas.SetFocus()
		self.window.SetSize ( (self.renderer.window_size[0], self.renderer.window_size[1]) )
		print('showing Frame')
		self.window.Show(True)
		
		print('calling SetTopWindow')
		self.SetTopWindow(self.window)
		
		self.Bind ( wx.EVT_CHAR, self.OnChar )
		self.canvas.Bind ( wx.EVT_SIZE, self.OnCanvasSize )
		self.canvas.Bind ( wx.EVT_PAINT, self.OnCanvasPaint )
		
		wx.IdleEvent.SetMode ( wx.IDLE_PROCESS_SPECIFIED )
		self.Bind ( wx.EVT_IDLE, self.OnIdle )
		
		print('making context current')
		self.canvas.SetCurrent ( self.context )
		self.renderer.init_gl()
	
	def OnCanvasPaint ( self, event ):
		self.render_scene()
	
	def render_scene(self):
		"render scene one time"
		self.canvas.SetCurrent ( self.context )
		self.renderer.render_scene()
		# Done rendering
		# self.canvas.SwapBuffers()
		if self.canvas.IsDoubleBuffered():
			self.canvas.SwapBuffers()
			print ("double buffered") # Do not want
		else:
			pass
			# TODO: SwapBuffers() seems required to show on desktop monitor,
			# but causes stalling when monitor is slower than VR headset
			self.canvas.SwapBuffers()
	
	def dispose_gl(self):
		pass
	
	def OnCanvasSize ( self, event ):
		wx.CallAfter(self.DoSetViewport)
		event.Skip()
	
	def DoSetViewport ( self ):
		size = self.canvas.GetClientSize()
		#print('DoSetViewport({},{})'.format(size.width,size.height))
		self.canvas.SetCurrent ( self.context )
		self.renderer.size_callback ( self.window, size.width, size.height )
	
	def OnChar ( self, event ):
		key = event.GetKeyCode()
		# print (key)
		if key == ord('q') or key == ord('Q') or key == wx.WXK_ESCAPE: # Q or ESCAPE
			# print ("closing")
			self.window.Close()
			sys.exit(0) # In non-debug mode, Frame.Close() does not seem to close the application
			return
		#self.window.Refresh(False)
		event.Skip()
	
	def run_loop ( self ):
		self.MainLoop()




#!/bin/env python

# file qt_pyside_app.py

import sys

from PyQt5.QtCore import Qt, QTimer
from PyQt5.QtWidgets import QApplication, QMainWindow
from PyQt5.QtOpenGL import QGLWidget, QGLFormat

"""
Toy PySide application for use with "hello world" examples demonstrating pyopenvr
"""


class MyGlWidget(QGLWidget):
    "PySideApp uses Qt library to create an opengl context, listen to keyboard events, and clean up"

    def __init__(self, renderer, glformat, app):
        "Creates an OpenGL context and a window, and acquires OpenGL resources"
        super(MyGlWidget, self).__init__(glformat)
        self.renderer = renderer
        self.app = app
        # Use a timer to rerender as fast as possible
        self.timer = QTimer(self)
        self.timer.setSingleShot(True)
        self.timer.setInterval(0)
        self.timer.timeout.connect(self.render_vr)
        # Accept keyboard events
        self.setFocusPolicy(Qt.StrongFocus)

    def __enter__(self):
        "setup for RAII using 'with' keyword"
        return self

    def __exit__(self, type_arg, value, traceback):
        "cleanup for RAII using 'with' keyword"
        self.dispose_gl()

    def initializeGL(self):
        if self.renderer is not None:
            self.renderer.init_gl()
        self.timer.start()

    def paintGL(self):
        "render scene one time"
        self.renderer.render_scene()
        self.swapBuffers() # Seems OK even in single-buffer mode
        
    def render_vr(self):
        self.makeCurrent()
        self.paintGL()
        self.doneCurrent()
        self.timer.start() # render again real soon now

    def disposeGL(self):
        if self.renderer is not None:
            self.makeCurrent()
            self.renderer.dispose_gl()
            self.doneCurrent()

    def keyPressEvent(self, event):
        "press ESCAPE to quit the application"
        key = event.key()
        if key == Qt.Key_Escape:
            self.app.quit()
            
            
class Qt5App(QApplication):
    def __init__(self, renderer, title):
        QApplication.__init__(self, sys.argv)
        self.window = QMainWindow()
        self.window.setWindowTitle(title)
        self.window.resize(800,600)
        # Get OpenGL 4.1 context
        glformat = QGLFormat()
        glformat.setVersion(4, 1)
        glformat.setProfile(QGLFormat.CoreProfile)
        glformat.setDoubleBuffer(False)
        self.glwidget = MyGlWidget(renderer, glformat, self)
        self.window.setCentralWidget(self.glwidget)
        self.window.show()
        
    def __enter__(self):
        "setup for RAII using 'with' keyword"
        return self

    def __exit__(self, type_arg, value, traceback):
        "cleanup for RAII using 'with' keyword"
        self.glwidget.disposeGL()

    def run_loop(self):
        retval = self.exec_()
        sys.exit(retval)


if __name__ == "__main__":
    from openvr.gl_renderer import OpenVrGlRenderer
    from openvr.color_cube_actor import ColorCubeActor
    actor = ColorCubeActor()
    renderer = OpenVrGlRenderer(actor)
    with Qt5App(renderer, "PySide OpenVR color cube") as qtPysideApp:
        qtPysideApp.run_loop()



import inspect
import textwrap


def shader_string(body, glsl_version='450 core'):
    """
    Call this method from a function that defines a literal shader string as the "body" argument.
    Dresses up a shader string in three ways:
        1) Insert #version at the top
        2) Insert #line number declaration
        3) un-indents
    The line number information can help debug glsl compile errors.
    The version string needs to be the very first characters in the shader,
    which can be distracting, requiring backslashes or other tricks.
    The unindenting allows you to type the shader code at a pleasing indent level
    in your python method, while still creating an unindented GLSL string at the end.
    """
    line_count = len(body.split('\n'))
    line_number = inspect.currentframe().f_back.f_lineno + 1 - line_count
    return """\
#version %s
%s
""" % (glsl_version, shader_substring(body, stack_frame=2))


def shader_substring(body, stack_frame=1):
    """
    Call this method from a function that defines a literal shader string as the "body" argument.
    Dresses up a shader string in two ways:
        1) Insert #line number declaration
        2) un-indents
    The line number information can help debug glsl compile errors.
    The unindenting allows you to type the shader code at a pleasing indent level
    in your python method, while still creating an unindented GLSL string at the end.
    """
    line_count = len(body.splitlines(True))
    line_number = inspect.stack()[stack_frame][2] + 1 - line_count
    return """\
#line %d
%s
""" % (line_number, textwrap.dedent(body))


if __name__ == "__main__":
    # Test example
    def vertex_shader():
        return shader_string("""
            layout(location = 0) in vec3 in_Position;
            layout(location = 1) in vec3 in_Normal;

            layout(location = 0) uniform mat4 projection = mat4(1);
            layout(location = 1) uniform mat4 model_view = mat4(1);

            out vec3 normal;

            void main() 
            {
                gl_Position = projection * model_view * vec4(in_Position, 1.0);
                mat4 normal_matrix = transpose(inverse(model_view));
                normal = normalize((normal_matrix * vec4(in_Normal, 0)).xyz);
            }        
            """)
    print(vertex_shader())



#!/bin/env python

# file color_cube_actor.py

from textwrap import dedent

from OpenGL.GL import *  # @UnusedWildImport # this comment squelches an IDE warning
from OpenGL.GL.shaders import compileShader, compileProgram

from openvr.glframework import shader_string

"""
Color cube for use in "hello world" openvr apps
"""


class ColorCubeActor(object):
    """
    Draws a cube
    
       2________ 3
       /|      /|
     6/_|____7/ |
      | |_____|_| 
      | /0    | /1
      |/______|/
      4       5
    """
    
    def __init__(self):
        self.shader = 0
        self.vao = None
    
    def init_gl(self):
        vertex_shader = compileShader(
            shader_string("""
            // Adapted from @jherico's RiftDemo.py in pyovr
            
            layout(location = 0) uniform mat4 Projection = mat4(1);
            layout(location = 4) uniform mat4 ModelView = mat4(1);
            layout(location = 8) uniform float Size = 0.3;
            
            // Minimum Y value is zero, so cube sits on the floor in room scale
            const vec3 UNIT_CUBE[8] = vec3[8](
              vec3(-1.0, -0.0, -1.0), // 0: lower left rear
              vec3(+1.0, -0.0, -1.0), // 1: lower right rear
              vec3(-1.0, +2.0, -1.0), // 2: upper left rear
              vec3(+1.0, +2.0, -1.0), // 3: upper right rear
              vec3(-1.0, -0.0, +1.0), // 4: lower left front
              vec3(+1.0, -0.0, +1.0), // 5: lower right front
              vec3(-1.0, +2.0, +1.0), // 6: upper left front
              vec3(+1.0, +2.0, +1.0)  // 7: upper right front
            );
            
            const vec3 UNIT_CUBE_NORMALS[6] = vec3[6](
              vec3(0.0, 0.0, -1.0),
              vec3(0.0, 0.0, 1.0),
              vec3(1.0, 0.0, 0.0),
              vec3(-1.0, 0.0, 0.0),
              vec3(0.0, 1.0, 0.0),
              vec3(0.0, -1.0, 0.0)
            );
            
            const int CUBE_INDICES[36] = int[36](
              0, 1, 2, 2, 1, 3, // front
              4, 6, 5, 6, 5, 7, // back
              0, 2, 4, 4, 2, 6, // left
              1, 3, 5, 5, 3, 7, // right
              2, 6, 3, 6, 3, 7, // top
              0, 1, 4, 4, 1, 5  // bottom
            );
            
            out vec3 _color;
            
            void main() {
              _color = vec3(1.0, 0.0, 0.0);
              int vertexIndex = CUBE_INDICES[gl_VertexID];
              int normalIndex = gl_VertexID / 6;
              
              _color = UNIT_CUBE_NORMALS[normalIndex];
              if (any(lessThan(_color, vec3(0.0)))) {
                  _color = vec3(1.0) + _color;
              }
            
              gl_Position = Projection * ModelView * vec4(UNIT_CUBE[vertexIndex] * Size, 1.0);
            }
            """), 
            GL_VERTEX_SHADER)
        fragment_shader = compileShader(
            shader_string("""
            in vec3 _color;
            out vec4 FragColor;
            
            void main() {
              FragColor = vec4(_color, 1.0);
            }
            """), 
            GL_FRAGMENT_SHADER)
        self.shader = compileProgram(vertex_shader, fragment_shader)
        #
        self.vao = glGenVertexArrays(1)
        glBindVertexArray(self.vao)
        glEnable(GL_DEPTH_TEST)
        
    def display_gl(self, modelview, projection):
        glUseProgram(self.shader)
        glUniformMatrix4fv(0, 1, False, projection)
        glUniformMatrix4fv(4, 1, False, modelview)
        glBindVertexArray(self.vao)
        glDrawArrays(GL_TRIANGLES, 0, 36)
    
    def dispose_gl(self):
        glDeleteProgram(self.shader)
        self.shader = 0
        if self.vao:
            glDeleteVertexArrays(1, (self.vao,))
        self.vao = 0



#!/bin/env python

# file glfwapp.py

from OpenGL.GL import *  # @UnusedWildImport # this comment squelches an IDE warning
from OpenGL.GLUT import * # @UnusedWildImport

"""
Toy glut application for use with "hello world" examples demonstrating pyopenvr
"""


class GlutApp(object):
    "GlutApp uses freeglut library to create an opengl context, listen to keyboard events, and clean up"

    def __init__(self, renderer, title=b"GLUT test"):
        "Creates an OpenGL context and a window, and acquires OpenGL resources"
        self.renderer = renderer
        self.title = title
        self.window = None
        self.init_gl()

    def __enter__(self):
        "setup for RAII using 'with' keyword"
        return self

    def __exit__(self, type_arg, value, traceback):
        "cleanup for RAII using 'with' keyword"
        self.dispose_gl()

    def init_gl(self):
        glutInit()
        glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_SINGLE)
        if bool(glutInitContextVersion):
            glutInitContextVersion(4, 1)
        # Create a regular desktop window, just so we can have an OpenGL context to play with
        glutInitWindowSize(800, 600)
        glutInitWindowPosition(50, 50)
        self.window = glutCreateWindow(self.title)
        # Set up callback methods for use during the GLUT main loop
        glutDisplayFunc(self.render_scene)
        glutIdleFunc(self.render_scene)
        glutReshapeFunc(self.resize_gl)
        glutKeyboardFunc(self.key_press)
        if self.renderer is not None:
            self.renderer.init_gl()

    def render_scene(self):
        "render scene one time"
        self.renderer.render_scene()
        # glutSwapBuffers() # double buffering stalls rendering
        glFlush()

    def dispose_gl(self):
        if self.window is not None:
            if self.renderer is not None:
                self.renderer.dispose_gl()
            self.window = None
        self._is_initialized = False

    def key_press(self, key, x, y):
        "Close the application when the player presses ESCAPE"
        if ord(key) == 27:
            # print "Escape!"
            if bool(glutLeaveMainLoop):
                glutLeaveMainLoop()
            else:
                raise Exception("Application quit")

    def resize_gl(self, width, height):
        "Called every time the on-screen window is resized"
        glViewport(0, 0, width, height)

    def run_loop(self):
        "keep rendering until the user says quit"
        glutMainLoop()



#!/bin/env python

# file sdl_app.py

import ctypes

from OpenGL.GL import glFlush  # @UnusedWildImport # this comment squelches an IDE warning
from sdl2 import * # @UnusedWildImport


"""
Minimal sdl programming example which colored OpenGL cube scene that can be closed by pressing ESCAPE.
"""

class SdlAppQuit(Exception):
	pass

class SdlApp(object):
	"SdlApp uses sdl2 library to create an opengl context, listen to keyboard events, and clean up"
	renderer = None
	title = None
	_is_initialized = False
	window = None
	context = None
	running = False
	_sdl_event_handlers = None

	def __init__(self, renderer, title="SDL test"):
		"Creates an OpenGL context and a window, and acquires OpenGL resources"
		self.renderer = renderer
		self.title = title
		self._is_initialized = False # keep track of whether self.init_gl() has been called
		self.window = None
		self._sdl_event_handlers = {
			SDL_WINDOWEVENT: self.on_sdl_window_event,
			SDL_KEYDOWN: self.on_sdl_keydown,
			SDL_QUIT: self.on_sdl_quit,
		}
	
	def __enter__(self):
		"setup for RAII using 'with' keyword"
		return self
	
	def __exit__(self, type_arg, value, traceback):
		"cleanup for RAII using 'with' keyword"
		print('SdlApp exiting')
		self.dispose_gl()
	
	def init_gl(self):
		if self._is_initialized:
			return # only initialize once
		result = SDL_Init(SDL_INIT_VIDEO)
		if result < 0:
			raise Exception("SDL Initialization error: {}".format(SDL_GetError()))
		# Get OpenGL 4.1 context
		SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 4)
		SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 1)
		SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE)
		SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 )
		SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 )
		SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 )
		SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 )
		SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 0 )
		self.window = SDL_CreateWindow (
			self.title.encode('utf-8'),
			SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
			self.renderer.window_size[0], self.renderer.window_size[1], SDL_WINDOW_SHOWN|SDL_WINDOW_OPENGL)
		if self.window is None:
			SDL_Quit()
			raise Exception("SDL window creation error: {}".format(SDL_GetError()))
		self.context = SDL_GL_CreateContext(self.window)
		SDL_GL_MakeCurrent ( self.window, self.context )
		if self.renderer is not None:
			self.renderer.init_gl()
		self._is_initialized = True
	
	def render_scene(self):
		"render scene one time"
		self.init_gl() # should be a no-op after the first frame is rendered
		SDL_GL_MakeCurrent ( self.window, self.context )
		self.renderer.render_scene()
		# Done rendering
		# SDL_GL_SwapWindow(self.window)
		glFlush()
	
	def dispose_gl(self):
		if self.window is not None:
			SDL_GL_MakeCurrent ( self.window, self.context )
			if self.renderer is not None:
				self.renderer.dispose_gl()
			if self.context is not None:
				SDL_GL_DeleteContext(self.context)
			SDL_DestroyWindow(self.window)
		SDL_Quit()
		self._is_initialized = False
	
	def on_sdl_window_event ( self, event ):
		if event.window.event == SDL_WINDOWEVENT_RESIZED:
			width, height = SDL_GL_GetDrawableSize ( self.window )
			self.renderer.size_callback ( self.window, width, height )
	
	def on_sdl_keydown ( self, event ):
		"press ESCAPE to quit the application"
		key = event.key.keysym.sym
		if key == SDLK_ESCAPE:
			self.running = False
	
	def on_sdl_quit ( self, event ):
		self.running = False
		raise SdlAppQuit()
	
	def run_loop(self):
		"keep rendering until the user says quit"
		self.running = True
		event = SDL_Event()
		try:
			while self.running:
				while SDL_PollEvent(ctypes.byref(event)) != 0:
					f = self._sdl_event_handlers.get(event.type)
					if f is not None:
						f ( event )
				self.render_scene()
		except SdlAppQuit as e:
			pass



# Store the version here so:
# 1) we don't load dependencies by storing it in __init__.py
# 2) we can import it in setup.py for the same reason
# 3) we can import it into your module module
# http://stackoverflow.com/questions/458550/standard-way-to-embed-version-into-python-package

__version__ = '1.16.0801'



#!/usr/bin/env python

from setuptools import setup

# Load module version from ovr/version.py
__version__ = '0.0.0'  # value will be replaced on the next line
exec(open('src/openvr/version.py').read())

setup(
    name='openvr',
    version=__version__,
    author='Christopher Bruns and others',
    author_email='cmbruns@rotatingpenguin.com',
    description='Unofficial python bindings for Valve OpenVR SDK',
    long_description='Valve OpenVR SDK python bindings using ctypes',
    url='https://github.com/cmbruns/pyopenvr',
    download_url='https://github.com/cmbruns/pyopenvr/tarball/' + __version__,
    package_dir={'': 'src'},
    packages=['openvr', 'openvr.glframework', 'openvr.error_code'],
    package_data={'openvr': ['*.dll', '*.so', '*.dylib']},
    keywords='openvr valve htc vive vr virtual reality 3d graphics',
    license='BSD',
    classifiers="""\
Environment :: Win32 (MS Windows)
Intended Audience :: Developers
License :: OSI Approved :: BSD License
Operating System :: Microsoft :: Windows
Operating System :: Microsoft :: Windows :: Windows 7
Operating System :: Microsoft :: Windows :: Windows 10
Operating System :: POSIX :: Linux
Operating System :: MacOS :: MacOS X
Programming Language :: Python :: 3.5
Programming Language :: Python :: 3.6
Programming Language :: Python :: Implementation :: CPython
Topic :: Multimedia :: Graphics :: 3D Rendering
Topic :: Scientific/Engineering :: Visualization
Development Status :: 4 - Beta
""".splitlines(),
    install_requires=[],
    extras_require={
        'glfw': ['numpy', 'PyOpenGL', 'glfw'],
        'opengl': ['numpy', 'PyOpenGL'],
        'PyQt5': ['numpy', 'PyOpenGL', 'PyQt5', 'jinja2'],
        'PySide2': ['numpy', 'PyOpenGL', 'PySide2'],
        'sdl2': ['numpy', 'PyOpenGL', 'PySDL2'],
        'wx': ['numpy', 'PyOpenGL', 'wxPython'],
        'generate': ['clang', ]
    }
)



#!/bin env python

import cyglfw3 as glfw

"""
Tow cyglfw3 application for use with "hello world" examples demonstrating pyopenvr
"""

class CyGLFW3App(object):
    """
    Uses the glfw library via the cyglfw3 bindings to create an opengl context, listen to keyboard
    and VR HMD/controller events, and clean up
    """

    def __init__(self, renderer, title="CyGLFW3 test"):
        self.renderer = renderer
        self.title = title
        self._is_initialized = False
        self.window = None

    def __enter__(self):
        """setup for 'with' keyword"""
        return self

    def __exit__(self, type_arg, value, traceback):
        """cleanup for 'with' keyword"""
        self.dispose_gl()

    def init_gl(self):
        if self._is_initialized:
            return
        if not glfw.Init():
            raise RuntimeError("GLFW Initialization failed")
        glfw.WindowHint(glfw.CONTEXT_VERSION_MAJOR, 4)
        glfw.WindowHint(glfw.CONTEXT_VERSION_MINOR, 1)
        glfw.WindowHint(glfw.OPENGL_PROFILE, glfw.OPENGL_CORE_PROFILE)
        glfw.WindowHint(glfw.DOUBLEBUFFER, False)
        glfw.SwapInterval(0)
        self.window = glfw.CreateWindow(self.renderer.window_size[0], self.renderer.window_size[1],
                                        self.title)
        if self.window is None:
            glfw.Terminate()
            raise RuntimeError("GLFW Window creation failed")
        glfw.SetKeyCallback(self.window, self.key_callback)
        glfw.MakeContextCurrent(self.window)
        if self.renderer is not None:
            self.renderer.init_gl()
        self._is_initialized = True

    def render_scene(self):
        "render scene one time"
        self.init_gl()
        glfw.MakeContextCurrent(self.window)
        self.renderer.render_scene()
        glfw.SwapBuffers(self.window)
        glfw.PollEvents()

    def dispose_gl(self):
        if self.window is not None:
            glfw.MakeContextCurrent(self.window)
            if self.renderer is not None:
                self.renderer.dispose_gl()
        glfw.Terminate()
        self._is_initialized = False

    def key_callback(self, window, key, scancode, action, mods):
        """press ESCAPE to quite the application"""
        if key == glfw.KEY_ESCAPE and action == glfw.PRESS:
            glfw.SetWindowShouldClose(self.window, True)

    def run_loop(self):
        """keep rendering until the user presses escape"""
        while not glfw.WindowShouldClose(self.window):
            self.render_scene()
